searchData={"items":[{"type":"module","title":"Bolt.Sips","doc":"A Neo4j driver for Elixir providing many useful features:\n\n- using the Bolt protocol, the Elixir implementation - the Neo4j's newest network protocol, designed for high-performance; latest Bolt versions, are supported.\n- Can connect to a standalone Neo4j server (`:direct` mode) or to a Neo4j causal cluster, using the `bolt+routing` or the newer `neo4j` schemes; connecting in `:routing` mode.\n- Provides the user with the ability to create and manage distinct ad-hoc `role-based` connections to one or more Neo4j servers/databases\n- Supports transactions, simple and complex Cypher queries with or w/o parameters\n- Multi-tenancy\n- Supports Neo4j versions: 3.0.x/3.1.x/3.2.x/3.4.x/3.5.x\n\nTo start, add the `:bolt_sips` dependency to you project, run `mix do deps.get, compile` on it and then you can quickly start experimenting with Neo4j from the convenience of your IEx shell. Example:\n\n    iex> {:ok, _neo} = Bolt.Sips.start_link(url: \"bolt://neo4j:test@localhost\")\n    {:ok, #PID }\n    iex>   conn = Bolt.Sips.conn()\n    #PID \n    iex> Bolt.Sips.query!(conn, \"RETURN 1 as n\")\n    %Bolt.Sips.Response{\n      records: [[1]],\n      results: [%{\"n\" => 1}]\n    }\n\nthe example above presumes that you have a Neo4j server available locally, using the Bolt protocol and requiring authentication.","ref":"Bolt.Sips.html"},{"type":"function","title":"Bolt.Sips.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Bolt.Sips.html#child_spec/1"},{"type":"function","title":"Bolt.Sips.conn/2","doc":"Returns a pool name which can be used to acquire a connection from a pool of servers\nresponsible with a specific type of operations: read, write and route, or all of the above: \"direct\"","ref":"Bolt.Sips.html#conn/2"},{"type":"function","title":"Bolt.Sips.info/0","doc":"peek into the main Router state, and return the internal state controlling the connections\nto the server/server. Mostly for internal use or for helping driver developers.\n\nThe authentication credentials will be sanitized, if any\n\n#","ref":"Bolt.Sips.html#info/0"},{"type":"function","title":"Examples: - Bolt.Sips.info/0","doc":"iex> Bolt.Sips.info()\n%{default: %{connections: %{direct: %{\"localhost:7687\" => 0}, routing_query: nil, zorba: %{\"localhost:7687\" => 0}}, user_options: [basic_auth: [username: \"******\", password: \"******\"], socket: Bolt.Sips.Socket, port: 7687, routing_context: %{}, schema: \"bolt\", hostname: \"localhost\", timeout: 15000, ssl: false, with_etls: false, prefix: :default, url: \"bolt://localhost\", pool_size: 10, max_overflow: 2, role: :zorba]}}","ref":"Bolt.Sips.html#info/0-examples"},{"type":"function","title":"Bolt.Sips.query/2","doc":"sends the query (and its parameters) to the server and returns `{:ok, Response.t()}` or\n`{:error, Error}` otherwise","ref":"Bolt.Sips.html#query/2"},{"type":"function","title":"Bolt.Sips.query/3","doc":"send a query and an associated map of parameters. Returns the server response or an error","ref":"Bolt.Sips.html#query/3"},{"type":"function","title":"Bolt.Sips.query/4","doc":"send a query and an associated map of parameters with options. Returns the server response or an error\nThe `opts` keyword list will be passed to `DbConnection.execute/4`.\nBy default, the query will timeout after `15_000` milliseconds, this may be overriden by setting the `timeout`\noption in `opts`.","ref":"Bolt.Sips.html#query/4"},{"type":"function","title":"Bolt.Sips.query!/2","doc":"The same as query/2 but raises a Exception if it fails.\nReturns the server response otherwise.","ref":"Bolt.Sips.html#query!/2"},{"type":"function","title":"Bolt.Sips.query!/3","doc":"The same as query/3 but raises a Exception if it fails.","ref":"Bolt.Sips.html#query!/3"},{"type":"function","title":"Bolt.Sips.query!/4","doc":"The same as query/4 but raises a Exception if it fails.","ref":"Bolt.Sips.html#query!/4"},{"type":"function","title":"Bolt.Sips.registry_name/0","doc":"the registry name used across the various driver components","ref":"Bolt.Sips.html#registry_name/0"},{"type":"function","title":"Bolt.Sips.rollback/2","doc":"Rollback a database transaction and release lock on connection.\n\nWhen inside of a `transaction/3` call does a non-local return, using a\n`throw/1` to cause the transaction to enter a failed state and the\n`transaction/3` call returns `{:error, reason}`. If `transaction/3` calls are\nnested the connection is marked as failed until the outermost transaction call\ndoes the database rollback.\n\n#","ref":"Bolt.Sips.html#rollback/2"},{"type":"function","title":"Example - Bolt.Sips.rollback/2","doc":"{:error, :oops} = Bolt.Sips.transaction(pool, fn(conn) ->\n      Bolt.Sips.rollback(conn, :oops)\n    end)","ref":"Bolt.Sips.html#rollback/2-example"},{"type":"function","title":"Bolt.Sips.routing_table/1","doc":"extract the routing table from the router","ref":"Bolt.Sips.html#routing_table/1"},{"type":"function","title":"Bolt.Sips.start_link/1","doc":"Start or add a new Neo4j connection","ref":"Bolt.Sips.html#start_link/1"},{"type":"function","title":"Options: - Bolt.Sips.start_link/1","doc":"- `:url`- a full url to pointing to a running Neo4j server. Please remember you must specify the scheme used to connect to the server. Valid schemes:`bolt`,`bolt+routing`and`neo4j` - the last two being used for connecting to a Neo4j causal cluster.\n- `:pool_size` - the size of the connection pool. Default: 15\n- `:timeout` - a connection timeout value defined in milliseconds. Default: 15_000\n- `:ssl`-`true`, if the connection must be encrypted. Default:`false`\n- `:prefix`- used for differentiating between multiple connections available in the same app. Default:`:default`","ref":"Bolt.Sips.html#start_link/1-options"},{"type":"function","title":"Example of valid configurations (i.e. defined in config/dev.exs) and usage: - Bolt.Sips.start_link/1","doc":"This is the most basic configuration:\n\n    config :bolt_sips, Bolt,\n      url: \"bolt://localhost:7687\"\n\nand if you need to connect to remote servers:\n\n    config :bolt_sips, Bolt,\n      url: \"bolt://Bilbo:Baggins@hobby-hobbits.dbs.graphenedb.com:24786\",\n      ssl: true,\n      timeout: 15_000,\n\nExample with a configuration defined in the `config/dev.exs`:\n\n    opts = Application.get_env(:bolt_sips, Bolt)\n    {:ok, pid} = Bolt.Sips.start_link(opts)\n\n    Bolt.Sips.query!(pid, \"CREATE (a:Person {name:'Bob'})\")\n    Bolt.Sips.query!(pid, \"MATCH (a:Person) RETURN a.name AS name\")\n    |> Enum.map(&(&1[\"name\"]))\n\nOr defining an ad-hoc configuration:\n\nExample with a configuration defined in the `config/dev.exs`:\n\n    {:ok, _neo} = Bolt.Sips.start_link(url: \"bolt://neo4j:test@localhost\")\n\n    conn = Bolt.Sips.conn()\n    Bolt.Sips.query!(conn, \"return 1 as n\")","ref":"Bolt.Sips.html#start_link/1-example-of-valid-configurations-i-e-defined-in-config-dev-exs-and-usage"},{"type":"function","title":"Bolt.Sips.terminate_connections/1","doc":"terminate a pool of connections with the role specified","ref":"Bolt.Sips.html#terminate_connections/1"},{"type":"function","title":"Bolt.Sips.transaction/3","doc":"Example:\n\n  ```elixir\n  setup do\n    {:ok, [main_conn: Bolt.Sips.conn()]}\n  end\n\n  test \"execute statements in transaction\", %{main_conn: main_conn} do\n    Bolt.Sips.transaction(main_conn, fn conn ->\n      book =\n        Bolt.Sips.query!(conn, \"CREATE (b:Book {title: \"The Game Of Trolls\"}) return b\")\n        |> Response.first()\n\n      assert %{\"b\" => g_o_t} = book\n      assert g_o_t.properties[\"title\"] == \"The Game Of Trolls\"\n      Bolt.Sips.rollback(conn, :changed_my_mind)\n    end)\n\n    books = Bolt.Sips.query!(main_conn, \"MATCH (b:Book {title: \"The Game Of Trolls\"}) return b\")\n    assert Enum.count(books) == 0\n  end\n  ```","ref":"Bolt.Sips.html#transaction/3"},{"type":"type","title":"Bolt.Sips.conn/0","doc":"","ref":"Bolt.Sips.html#t:conn/0"},{"type":"type","title":"Bolt.Sips.transaction/0","doc":"","ref":"Bolt.Sips.html#t:transaction/0"},{"type":"module","title":"Bolt.Sips.BoltProtocol.Message.HelloMessage","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.HelloMessage.html"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.HelloMessage.decode/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.HelloMessage.html#decode/1"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.HelloMessage.encode/2","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.HelloMessage.html#encode/2"},{"type":"module","title":"Bolt.Sips.BoltProtocol.Message.InitMessage","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.InitMessage.html"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.InitMessage.decode/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.InitMessage.html#decode/1"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.InitMessage.encode/2","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.InitMessage.html#encode/2"},{"type":"module","title":"Bolt.Sips.BoltProtocol.Message.LogonMessage","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.LogonMessage.html"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.LogonMessage.decode/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.LogonMessage.html#decode/1"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.LogonMessage.encode/2","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.LogonMessage.html#encode/2"},{"type":"module","title":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.html"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.get_auth_params/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.html#get_auth_params/1"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.get_bolt_agent/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.html#get_bolt_agent/1"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.get_user_agent/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.html#get_user_agent/1"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.get_user_agent/2","doc":"","ref":"Bolt.Sips.BoltProtocol.Message.Shared.AuthHelper.html#get_user_agent/2"},{"type":"module","title":"Bolt.Sips.BoltProtocol.Versions","doc":"","ref":"Bolt.Sips.BoltProtocol.Versions.html"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Versions.available_versions/0","doc":"","ref":"Bolt.Sips.BoltProtocol.Versions.html#available_versions/0"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Versions.latest_versions/0","doc":"","ref":"Bolt.Sips.BoltProtocol.Versions.html#latest_versions/0"},{"type":"function","title":"Bolt.Sips.BoltProtocol.Versions.to_bytes/1","doc":"","ref":"Bolt.Sips.BoltProtocol.Versions.html#to_bytes/1"},{"type":"module","title":"Bolt.Sips.Client","doc":"","ref":"Bolt.Sips.Client.html"},{"type":"function","title":"Bolt.Sips.Client.checkin/1","doc":"","ref":"Bolt.Sips.Client.html#checkin/1"},{"type":"function","title":"Bolt.Sips.Client.connect/1","doc":"","ref":"Bolt.Sips.Client.html#connect/1"},{"type":"function","title":"Bolt.Sips.Client.disconnect/1","doc":"","ref":"Bolt.Sips.Client.html#disconnect/1"},{"type":"function","title":"Bolt.Sips.Client.do_connect/1","doc":"","ref":"Bolt.Sips.Client.html#do_connect/1"},{"type":"function","title":"Bolt.Sips.Client.message_hello/2","doc":"","ref":"Bolt.Sips.Client.html#message_hello/2"},{"type":"function","title":"Bolt.Sips.Client.message_init/2","doc":"","ref":"Bolt.Sips.Client.html#message_init/2"},{"type":"function","title":"Bolt.Sips.Client.message_logon/2","doc":"","ref":"Bolt.Sips.Client.html#message_logon/2"},{"type":"function","title":"Bolt.Sips.Client.recv_data/2","doc":"","ref":"Bolt.Sips.Client.html#recv_data/2"},{"type":"function","title":"Bolt.Sips.Client.recv_packets/2","doc":"","ref":"Bolt.Sips.Client.html#recv_packets/2"},{"type":"function","title":"Bolt.Sips.Client.recv_packets/3","doc":"","ref":"Bolt.Sips.Client.html#recv_packets/3"},{"type":"function","title":"Bolt.Sips.Client.send_data/2","doc":"","ref":"Bolt.Sips.Client.html#send_data/2"},{"type":"function","title":"Bolt.Sips.Client.send_packet/2","doc":"","ref":"Bolt.Sips.Client.html#send_packet/2"},{"type":"module","title":"Bolt.Sips.Error","doc":"represents an error message","ref":"Bolt.Sips.Error.html"},{"type":"function","title":"Bolt.Sips.Error.new/1","doc":"","ref":"Bolt.Sips.Error.html#new/1"},{"type":"type","title":"Bolt.Sips.Error.t/0","doc":"","ref":"Bolt.Sips.Error.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Internals.BoltProtocolV3","doc":"","ref":"Bolt.Sips.Internals.BoltProtocolV3.html"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.begin/5","doc":"Implementation of Bolt's BEGIN. It opens a transaction.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#begin/5"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.begin/5","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#begin/5-options"},{"type":"function","title":"Example - Bolt.Sips.Internals.BoltProtocolV3.begin/5","doc":"iex> BoltProtocolV3.begin(:gen_tcp, port, 3, [])\n    {:ok, metadata}","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#begin/5-example"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.commit/4","doc":"Implementation of Bolt's COMMIT. It commits the open transaction.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#commit/4"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.commit/4","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#commit/4-options"},{"type":"function","title":"Example - Bolt.Sips.Internals.BoltProtocolV3.commit/4","doc":"iex> BoltProtocolV3.commit(:gen_tcp, port, 3, [])\n    :ok","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#commit/4-example"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.goodbye/3","doc":"Implementation of Bolt's RUN. It closes the connection.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#goodbye/3"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.goodbye/3","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#goodbye/3-options"},{"type":"function","title":"Examples - Bolt.Sips.Internals.BoltProtocolV3.goodbye/3","doc":"iex> Bolt.Sips.Internals.BoltProtocolV3.goodbye(:gen_tcp, port, 3)\n    :ok\n\n    iex> Bolt.Sips.Internals.BoltProtocolV3.goodbye(:gen_tcp, port, 3)\n    :ok","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#goodbye/3-examples"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.hello/5","doc":"Implementation of Bolt's HELLO. It initialises the connection.\n\nExpects a transport module (i.e. `gen_tcp`) and a `Port`. Accepts\nauthorisation params in the form of {username, password}.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#hello/5"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.hello/5","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#hello/5-options"},{"type":"function","title":"Examples - Bolt.Sips.Internals.BoltProtocolV3.hello/5","doc":"iex> Bolt.Sips.Internals.BoltProtocolV3.hello(:gen_tcp, port, 3, {}, [])\n    {:ok, info}\n\n    iex> Bolt.Sips.Internals.BoltProtocolV3.hello(:gen_tcp, port, 3, {\"username\", \"password\"}, [])\n    {:ok, info}","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#hello/5-examples"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.rollback/4","doc":"Implementation of Bolt's ROLLBACK. It rollbacks the open transaction.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#rollback/4"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.rollback/4","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#rollback/4-options"},{"type":"function","title":"Example - Bolt.Sips.Internals.BoltProtocolV3.rollback/4","doc":"iex> BoltProtocolV3.rollback(:gen_tcp, port, 3, [])\n    :ok","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#rollback/4-example"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.run/7","doc":"Implementation of Bolt's RUN. It passes a statement for execution on the server.\n\nNote that this message doesn't return the statement result. For this purpose, use PULL_ALL.\nIn bolt >= 3, run has an additional paramters; metadata","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#run/7"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.run/7","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#run/7-options"},{"type":"function","title":"Example - Bolt.Sips.Internals.BoltProtocolV3.run/7","doc":"iex> BoltProtocolV1.run(:gen_tcp, port, 1, \"RETURN $num AS num\", %{num: 5}, %{}, [])\n    {:ok, {:success, %{\"fields\" => [\"num\"]}}}","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#run/7-example"},{"type":"function","title":"Bolt.Sips.Internals.BoltProtocolV3.run_statement/7","doc":"Runs a statement (most likely Cypher statement) and returns a list of the\nrecords and a summary (Act as as a RUN + PULL_ALL).\n\nRecords are represented using PackStream's record data type. Their Elixir\nrepresentation is a Keyword with the indexes `:sig` and `:fields`.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#run_statement/7"},{"type":"function","title":"Options - Bolt.Sips.Internals.BoltProtocolV3.run_statement/7","doc":"See \"Shared options\" in `Bolt.Sips.Internals.BoltProtocolHelper` documentation.","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#run_statement/7-options"},{"type":"function","title":"Examples - Bolt.Sips.Internals.BoltProtocolV3.run_statement/7","doc":"iex> Bolt.Sips.Internals.BoltProtocol.run_statement(:gen_tcp, port, 1, \"MATCH (n) RETURN n\")\n    [\n      {:success, %{\"fields\" => [\"n\"]}},\n      {:record, [sig: 1, fields: [1, \"Example\", \"Labels\", %{\"some_attribute\" => \"some_value\"}]]},\n      {:success, %{\"type\" => \"r\"}}\n    ]","ref":"Bolt.Sips.Internals.BoltProtocolV3.html#run_statement/7-examples"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.DecoderImplV1","doc":"","ref":"Bolt.Sips.Internals.PackStream.DecoderImplV1.html"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.DecoderImplV2","doc":"","ref":"Bolt.Sips.Internals.PackStream.DecoderImplV2.html"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.DecoderUtils","doc":"","ref":"Bolt.Sips.Internals.PackStream.DecoderUtils.html"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.DecoderV3","doc":"","ref":"Bolt.Sips.Internals.PackStream.DecoderV3.html"},{"type":"function","title":"Bolt.Sips.Internals.PackStream.DecoderV3.decode/2","doc":"","ref":"Bolt.Sips.Internals.PackStream.DecoderV3.html#decode/2"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.EncoderV3","doc":"","ref":"Bolt.Sips.Internals.PackStream.EncoderV3.html"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.Message.EncoderV2","doc":"","ref":"Bolt.Sips.Internals.PackStream.Message.EncoderV2.html"},{"type":"function","title":"Bolt.Sips.Internals.PackStream.Message.EncoderV2.encode/2","doc":"","ref":"Bolt.Sips.Internals.PackStream.Message.EncoderV2.html#encode/2"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.Utils","doc":"","ref":"Bolt.Sips.Internals.PackStream.Utils.html"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.V1","doc":"","ref":"Bolt.Sips.Internals.PackStream.V1.html"},{"type":"module","title":"Bolt.Sips.Internals.PackStream.V2","doc":"","ref":"Bolt.Sips.Internals.PackStream.V2.html"},{"type":"module","title":"Bolt.Sips.LoadBalancer","doc":"a simple load balancer used for selecting a server address from a map. The address is selected based on\nhow many hits has; least reused url.","ref":"Bolt.Sips.LoadBalancer.html"},{"type":"function","title":"Bolt.Sips.LoadBalancer.least_reused_url/1","doc":"sort by total number of hits and return the least reused url","ref":"Bolt.Sips.LoadBalancer.html#least_reused_url/1"},{"type":"function","title":"Examples - Bolt.Sips.LoadBalancer.least_reused_url/1","doc":"iex> least_reused_url(%{\"url1\" => 10, \"url2\" => 5})\n    {:ok, \"url2\"}\n\n    iex> least_reused_url(%{})\n    {:error, :not_found}","ref":"Bolt.Sips.LoadBalancer.html#least_reused_url/1-examples"},{"type":"module","title":"Bolt.Sips.Query","doc":"Provides a simple Query DSL.\n\nYou can run simple Cypher queries with or w/o parameters, for example:\n\n    {:ok, row} = Bolt.Sips.query(conn, \"match (n:Person {bolt_sips: true}) return n.name as Name limit 5\")\n    assert List.first(row)[\"Name\"] == \"Patrick Rothfuss\"\n\nOr more complex ones:\n\n    cypher = \"\"\"\n    MATCH (p:Person {bolt_sips: true})\n    RETURN p, p.name AS name, upper(p.name) as NAME,\n           coalesce(p.nickname,\"n/a\") AS nickname,\n           { name: p.name, label:head(labels(p))} AS person\n    \"\"\"\n    {:ok, r} = Bolt.Sips.query(conn, cypher)\n\nAs you can see, you can organize your longer queries using the Elixir multiple line conventions, for readability.\n\nAnd there is one more trick, you can use for more complex Cypher commands: use `;` as a transactional separator.\n\nFor example, say you want to clean up the test database **before** creating some tests entities. You can do that like this:\n\n    cypher = \"\"\"\n    MATCH (n {bolt_sips: true}) OPTIONAL MATCH (n)-[r]-() DELETE n,r;\n\n    CREATE (BoltSips:BoltSips {title:'Elixir sipping from Neo4j, using Bolt', released:2016, license:'MIT', bolt_sips: true})\n    CREATE (TNOTW:Book {title:'The Name of the Wind', released:2007, genre:'fantasy', bolt_sips: true})\n    CREATE (Patrick:Person {name:'Patrick Rothfuss', bolt_sips: true})\n    CREATE (Kvothe:Person {name:'Kote', bolt_sips: true})\n    CREATE (Denna:Person {name:'Denna', bolt_sips: true})\n    CREATE (Chandrian:Deamon {name:'Chandrian', bolt_sips: true})\n\n    CREATE\n      (Kvothe)-[:ACTED_IN {roles:['sword fighter', 'magician', 'musician']}]->(TNOTW),\n      (Denna)-[:ACTED_IN {roles:['many talents']}]->(TNOTW),\n      (Chandrian)-[:ACTED_IN {roles:['killer']}]->(TNOTW),\n      (Patrick)-[:WROTE]->(TNOTW)\n    \"\"\"\n    assert {:ok, _r} = Bolt.Sips.query(conn, cypher)\n\nIn the example above, this command: `MATCH (n {bolt_sips: true}) OPTIONAL MATCH (n)-[r]-() DELETE n,r;` will be executed in a distinct transaction, before all the other queries\n\nSee the various tests, or more examples and implementation details.","ref":"Bolt.Sips.Query.html"},{"type":"function","title":"Bolt.Sips.Query.query/2","doc":"","ref":"Bolt.Sips.Query.html#query/2"},{"type":"function","title":"Bolt.Sips.Query.query/4","doc":"","ref":"Bolt.Sips.Query.html#query/4"},{"type":"function","title":"Bolt.Sips.Query.query!/2","doc":"","ref":"Bolt.Sips.Query.html#query!/2"},{"type":"function","title":"Bolt.Sips.Query.query!/4","doc":"","ref":"Bolt.Sips.Query.html#query!/4"},{"type":"module","title":"Bolt.Sips.Response","doc":"Support for transforming a Bolt response to a list of Bolt.Sips.Types or arbitrary values.\nA Bolt.Sips.Response is used for mapping any response received from a Neo4j server into a an Elixir struct.\n\nYou'll interact with this module every time you're needing to get and manipulate the data resulting from your queries.\n\nFor example, a simple Cypher query like this:\n\n    Bolt.Sips.query(Bolt.Sips.conn(), \"RETURN [10,11,21] AS arr\")\n\n\nwill return:\n\n    {:ok,\n     %Bolt.Sips.Response{\n      bookmark: \"neo4j:bookmark:v1:tx21868\",\n      fields: [\"arr\"],\n      notifications: [],\n      plan: nil,\n      profile: nil,\n      records: [[[10, 11, 21]]],\n      results: [%{\"arr\" => [10, 11, 21]}],\n      stats: [],\n      type: \"r\"\n     }}\n\nand while you have now access to the full data returned by Neo4j, most of the times you'll just want the results:\n\n    iex> %Bolt.Sips.Response{results: results} = Bolt.Sips.query!(Bolt.Sips.conn(), \"RETURN [10,11,21] AS arr\")\n    iex> results\n    [%{\"arr\" => [10, 11, 21]}]\n\nMore complex queries, i.e.:\n\n    MATCH p=({name:'Alice'})-[r:KNOWS]->({name:'Bob'}) RETURN r\n\nwill return `results` like this:\n\n    [%{\"r\" => %Bolt.Sips.Types.Relationship{end: 647, id: 495,\n     properties: %{}, start: 646, type: \"KNOWS\"}}]\n\nNote: the `Path` has also functionality for \"drawing\" a graph, from a given node-relationship path\n\nOur Bolt.Sips.Response is implementing Elixir's [Enumerable Protocol](https://hexdocs.pm/elixir/Enumerable.html), to help you accessing the results. Hence something like this, is possible:\n\n    iex> Bolt.Sips.query!(Bolt.Sips.conn(), \"RETURN [10,11,21] AS arr\") |>\n    ...> Enum.reduce(0, &(Enum.sum(&1[\"arr\"]) + &2))\n    42\n\nan overly complicated example, but you get the point?! :)\n\nYou can also quickly get the `first` of the results, returned by Neo4j. Example:\n\n    iex> Bolt.Sips.query!(Bolt.Sips.conn(), \"UNWIND range(1, 10) AS n RETURN n\") |>\n    ...> Bolt.Sips.Response.first()\n    %{\"n\" => 1}","ref":"Bolt.Sips.Response.html"},{"type":"function","title":"Bolt.Sips.Response.fetch/2","doc":"","ref":"Bolt.Sips.Response.html#fetch/2"},{"type":"function","title":"Bolt.Sips.Response.fetch!/2","doc":"","ref":"Bolt.Sips.Response.html#fetch!/2"},{"type":"function","title":"Bolt.Sips.Response.first/1","doc":"","ref":"Bolt.Sips.Response.html#first/1"},{"type":"function","title":"Bolt.Sips.Response.transform/2","doc":"","ref":"Bolt.Sips.Response.html#transform/2"},{"type":"type","title":"Bolt.Sips.Response.acc/0","doc":"","ref":"Bolt.Sips.Response.html#t:acc/0"},{"type":"type","title":"Bolt.Sips.Response.element/0","doc":"","ref":"Bolt.Sips.Response.html#t:element/0"},{"type":"type","title":"Bolt.Sips.Response.key/0","doc":"","ref":"Bolt.Sips.Response.html#t:key/0"},{"type":"type","title":"Bolt.Sips.Response.t/0","doc":"","ref":"Bolt.Sips.Response.html#t:t/0"},{"type":"type","title":"Bolt.Sips.Response.value/0","doc":"","ref":"Bolt.Sips.Response.html#t:value/0"},{"type":"module","title":"Bolt.Sips.ResponseEncoder","doc":"This module provides functions to encode a query result or data containing Bolt.Sips.Types\ninto various format.\n\nFor now, only JSON is supported.\n\nEncoding is  handled by protocols to allow override if a  specific implemention is required.\nSee targeted protocol documentation for more information","ref":"Bolt.Sips.ResponseEncoder.html"},{"type":"function","title":"Bolt.Sips.ResponseEncoder.encode/2","doc":"Encode the data in json format.\n\nThis is done is 2 steps:\n- first, the data is converted into a jsonable format\n- the result is encoded in json via Jason\n\nBoth of these steps are overridable, see:\n- for step 1: `Bolt.Sips.ResponseEncoder.Json`\n- for step 2 (depending of your preferred library):\n- `Bolt.Sips.ResponseEncoder.Json.Jason`\n- `Bolt.Sips.ResponseEncoder.Json.Poison`","ref":"Bolt.Sips.ResponseEncoder.html#encode/2"},{"type":"function","title":"Example - Bolt.Sips.ResponseEncoder.encode/2","doc":"iex> data = %{\"t1\" => %Bolt.Sips.Types.Node{\n    ...>     id: 69,\n    ...>     labels: [\"Test\"],\n    ...>     properties: %{\n    ...>       \"created\" => %Bolt.Sips.Types.DateTimeWithTZOffset{\n    ...>         naive_datetime: ~N[2016-05-24 13:26:08.543],\n    ...>         timezone_offset: 7200\n    ...>       },\n    ...>       \"uuid\" => 12345\n    ...>     }\n    ...>   }\n    ...> }\n    iex> Bolt.Sips.ResponseEncoder.encode(data, :json)\n    {:ok, ~S|{\"t1\":{\"id\":69,\"labels\":[\"Test\"],\"properties\":{\"created\":\"2016-05-24T13:26:08.543+02:00\",\"uuid\":12345}}}|}\n\n    iex> Bolt.Sips.ResponseEncoder.encode(\"\\xFF\", :json)\n    {:error, %Jason.EncodeError{message: \"invalid byte 0xFF in  >\"}}","ref":"Bolt.Sips.ResponseEncoder.html#encode/2-example"},{"type":"function","title":"Bolt.Sips.ResponseEncoder.encode!/2","doc":"Encode the data in json format.\n\nSimilar to `encode/1` except it will unwrap the error tuple and raise in case of errors.","ref":"Bolt.Sips.ResponseEncoder.html#encode!/2"},{"type":"function","title":"Example - Bolt.Sips.ResponseEncoder.encode!/2","doc":"iex> data = %{\"t1\" => %Bolt.Sips.Types.Node{\n    ...>     id: 69,\n    ...>     labels: [\"Test\"],\n    ...>     properties: %{\n    ...>       \"created\" => %Bolt.Sips.Types.DateTimeWithTZOffset{\n    ...>         naive_datetime: ~N[2016-05-24 13:26:08.543],\n    ...>         timezone_offset: 7200\n    ...>       },\n    ...>       \"uuid\" => 12345\n    ...>     }\n    ...>   }\n    ...> }\n    iex> Bolt.Sips.ResponseEncoder.encode!(data, :json)\n    ~S|{\"t1\":{\"id\":69,\"labels\":[\"Test\"],\"properties\":{\"created\":\"2016-05-24T13:26:08.543+02:00\",\"uuid\":12345}}}|\n\n    iex> Bolt.Sips.ResponseEncoder.encode!(\"\\xFF\", :json)\n    ** (Jason.EncodeError) invalid byte 0xFF in  >","ref":"Bolt.Sips.ResponseEncoder.html#encode!/2-example"},{"type":"protocol","title":"Bolt.Sips.ResponseEncoder.Json","doc":"Protocol controlling how a value is made jsonable.\n\nIts only purpose is to convert Bolt Sips specific structures into elixir buit-in types\nwhich can be encoed in json by Jason.","ref":"Bolt.Sips.ResponseEncoder.Json.html"},{"type":"protocol","title":"Deriving - Bolt.Sips.ResponseEncoder.Json","doc":"If the provided default implementation don't fit your need, you can override with your own\nimplementation.\n\n#","ref":"Bolt.Sips.ResponseEncoder.Json.html#module-deriving"},{"type":"protocol","title":"Example - Bolt.Sips.ResponseEncoder.Json","doc":"Let's assume that you don't want Node's id available as they are Neo4j's ones and are not\nreliable because of id reuse and you want to have you own `uuid` in place.\nInstead of:\n```\n  {\n    id: 0,\n    labels: [\"TestNode\"],\n    properties: {\n      uuid: \"837806a7-6c37-4630-9f6c-9aa7ad0129ed\"\n      value: \"my node\"\n    }\n  }\n```\nyou want:\n```\n  {\n    uuid: \"837806a7-6c37-4630-9f6c-9aa7ad0129ed\",\n    labels: [\"TestNode\"],\n    properties: {\n      value: \"my node\"\n    }\n  }\n```\n\nYou can achieve that with the following implementation:\n```\ndefimpl Bolt.Sips.ResponseEncoder.Json, for: Bolt.Sips.Types.Node do\n  def encode(node) do\n    new_props = Map.drop(node.properties, :uuid)\n\n    node\n    |> Map.from_struct()\n    |> Map.put(:uuid, node.properties.uuid)\n    |> Map.put(:properties, new_props)\n  end\nend\n```\n\nIt is also possible to provide implementation that returns structs or updated Bolt.Sips.Types,\nthe use of a final `Bolt.Sips.ResponseEncoder.Json.encode()` will ensure that these values will\nbe converted to jsonable ones.","ref":"Bolt.Sips.ResponseEncoder.Json.html#module-example"},{"type":"function","title":"Bolt.Sips.ResponseEncoder.Json.encode/1","doc":"Convert a value in a jsonable format","ref":"Bolt.Sips.ResponseEncoder.Json.html#encode/1"},{"type":"type","title":"Bolt.Sips.ResponseEncoder.Json.t/0","doc":"All the types that implement this protocol.","ref":"Bolt.Sips.ResponseEncoder.Json.html#t:t/0"},{"type":"module","title":"Bolt.Sips.ResponseEncoder.Json.Jason","doc":"A default implementation for Jason encoding library.\n\nMore info about Jason: [https://hex.pm/packages/jason](https://hex.pm/packages/jason)\n\nAllow this usage:\n```\nconn = Bolt.Sips.conn()\n{:ok, res} = Bolt.Sips.query(conn, \"MATCH (t:TestNode) RETURN t\")\nJason.encode!(res)\n```\n\nDefault implementation can be overriden by providing your own implementation.\n\nMore info about implementation: [https://hexdocs.pm/jason/readme.html#differences-to-poison](https://hexdocs.pm/jason/readme.html#differences-to-poison)\n\n##","ref":"Bolt.Sips.ResponseEncoder.Json.Jason.html"},{"type":"module","title":"Note: - Bolt.Sips.ResponseEncoder.Json.Jason","doc":"In order to benefit from Bolt.Sips.ResponseEncoder implementation, use\n`Bolt.Sips.ResponseEncoder.Json.encode` and pass the result to the Jason\nencoding functions.","ref":"Bolt.Sips.ResponseEncoder.Json.Jason.html#module-note"},{"type":"module","title":"Bolt.Sips.ResponseEncoder.Json.Poison","doc":"A default implementation for Poison encoding library.\nMore info about poison here: [https://hex.pm/packages/poison](https://hex.pm/packages/poison)\n\nAllow this usage:\n```\nconn = Bolt.Sips.conn()\n{:ok, res} = Bolt.Sips.query(conn, \"MATCH (t:TestNode) RETURN t\")\nPoison.encode!(res)\n```\n\nDefault implementation can be overriden by providing your own implementation.\n\nMore info about implementation: [https://hexdocs.pm/poison/Poison.html#module-encoder](https://hexdocs.pm/poison/Poison.html#module-encoder)\n\n##","ref":"Bolt.Sips.ResponseEncoder.Json.Poison.html"},{"type":"module","title":"Note: - Bolt.Sips.ResponseEncoder.Json.Poison","doc":"In order to benefit from Bolt.Sips.ResponseEncoder implementation, use\n`Bolt.Sips.ResponseEncoder.Json.encode` and pass the result to the Poison\nencoding functions.","ref":"Bolt.Sips.ResponseEncoder.Json.Poison.html#module-note"},{"type":"module","title":"Bolt.Sips.Router","doc":"This \"driver\" works in tandem with Neo4j's [Causal Clustering](https://neo4j.com/docs/operations-manual/current/clustering/>) feature by directing read and write behaviour to appropriate cluster members","ref":"Bolt.Sips.Router.html"},{"type":"function","title":"Bolt.Sips.Router.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Bolt.Sips.Router.html#child_spec/1"},{"type":"function","title":"Bolt.Sips.Router.configure/1","doc":"","ref":"Bolt.Sips.Router.html#configure/1"},{"type":"function","title":"Bolt.Sips.Router.get_connection/2","doc":"","ref":"Bolt.Sips.Router.html#get_connection/2"},{"type":"function","title":"Bolt.Sips.Router.info/0","doc":"","ref":"Bolt.Sips.Router.html#info/0"},{"type":"function","title":"Bolt.Sips.Router.merge_connections_maps/3","doc":"","ref":"Bolt.Sips.Router.html#merge_connections_maps/3"},{"type":"function","title":"Bolt.Sips.Router.parse_server_version/1","doc":"parse the version string received from the server, while considering the lack of the\npatch number in some situations","ref":"Bolt.Sips.Router.html#parse_server_version/1"},{"type":"function","title":"Examples - Bolt.Sips.Router.parse_server_version/1","doc":"iex> Bolt.Sips.Router.parse_server_version(%{\"server\" => \"Neo4j/3.5.0\"})\n{\"Neo4j/3.5.0\", \"3.5.0\"}\n\niex> Bolt.Sips.Router.parse_server_version(%{\"server\" => \"Neo4j/3.5\"})\n{\"Neo4j/3.5\", \"3.5.0\"}\n\niex> Bolt.Sips.Router.parse_server_version(%{\"server\" => \"Neo4j/3.5.10\"})\n{\"Neo4j/3.5.10\", \"3.5.10\"}\n\niex> Bolt.Sips.Router.parse_server_version(%{\"server\" => \"Neo4j/3.5.11.1\"})\n{\"Neo4j/3.5.11.1\", \"3.5.11\"}","ref":"Bolt.Sips.Router.html#parse_server_version/1-examples"},{"type":"function","title":"Bolt.Sips.Router.routing_table/1","doc":"","ref":"Bolt.Sips.Router.html#routing_table/1"},{"type":"function","title":"Bolt.Sips.Router.start_connections/2","doc":"start a new (DB)Connection process, supervised registered under a name following this convention:\n\n- \"role@hostname:port\", the `role`, `hostname` and the `port` are collected from the user's\n configuration: `opts`. The `role` parameter is ignored when the `routing_table` parameter represents\n a neo4j map containing the definition for a neo4j cluster! It defaults to: `:direct`, when not specified!","ref":"Bolt.Sips.Router.html#start_connections/2"},{"type":"function","title":"Bolt.Sips.Router.start_link/1","doc":"","ref":"Bolt.Sips.Router.html#start_link/1"},{"type":"function","title":"Bolt.Sips.Router.terminate_connections/2","doc":"","ref":"Bolt.Sips.Router.html#terminate_connections/2"},{"type":"module","title":"Bolt.Sips.Router.State","doc":"todo:\nthis is work in progress and will be used for defining the state of the Router (Gen)Server","ref":"Bolt.Sips.Router.State.html"},{"type":"type","title":"Bolt.Sips.Router.State.role/0","doc":"","ref":"Bolt.Sips.Router.State.html#t:role/0"},{"type":"type","title":"Bolt.Sips.Router.State.t/0","doc":"","ref":"Bolt.Sips.Router.State.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Routing.RoutingTable","doc":"representing the routing table elements\n\nThere are a couple of ways to get the routing table from the server, for recent Neo4j servers, and with the\nlatest version of Bolt.Sips, you could use this query:\n\n    Bolt.Sips.query!(Bolt.Sips.conn, \"call dbms.cluster.routing.getRoutingTable($props)\", %{props: %{}})\n\n    [\n      %{\n        \"servers\" => [\n          %{\"addresses\" => [\"localhost:7687\"], \"role\" => \"WRITE\"},\n          %{\"addresses\" => [\"localhost:7689\", \"localhost:7688\"], \"role\" => \"READ\"},\n          %{\n            \"addresses\" => [\"localhost:7688\", \"localhost:7689\", \"localhost:7687\"],\n            \"role\" => \"ROUTE\"\n          }\n        ],\n        \"ttl\" => 300\n      }\n    ]","ref":"Bolt.Sips.Routing.RoutingTable.html"},{"type":"function","title":"Bolt.Sips.Routing.RoutingTable.parse/1","doc":"","ref":"Bolt.Sips.Routing.RoutingTable.html#parse/1"},{"type":"function","title":"Bolt.Sips.Routing.RoutingTable.parse_ttl/1","doc":"","ref":"Bolt.Sips.Routing.RoutingTable.html#parse_ttl/1"},{"type":"type","title":"Bolt.Sips.Routing.RoutingTable.t/0","doc":"","ref":"Bolt.Sips.Routing.RoutingTable.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Socket","doc":"A default socket interface used to communicate to a Neo4j instance.\n\nAny other socket implementing the same interface can be used\nin place of this one. Actually, this module doesn't\nimplement the interface on its own, it delegates calls to\nthe gen_tcp (http://erlang.org/doc/man/gen_tcp.html)\nand inet (http://erlang.org/doc/man/inet.html) modules. Any of these\nmodules doesn't fully implement the required interface,\nhence, both of them must be used.","ref":"Bolt.Sips.Socket.html"},{"type":"function","title":"Bolt.Sips.Socket.close/1","doc":"Delegates to :inet.close/1","ref":"Bolt.Sips.Socket.html#close/1"},{"type":"function","title":"Bolt.Sips.Socket.connect/4","doc":"Delegates to :gen_tcp.connect/4","ref":"Bolt.Sips.Socket.html#connect/4"},{"type":"function","title":"Bolt.Sips.Socket.recv/2","doc":"Delegates to :gen_tcp.recv/2","ref":"Bolt.Sips.Socket.html#recv/2"},{"type":"function","title":"Bolt.Sips.Socket.recv/3","doc":"Delegates to :gen_tcp.recv/3","ref":"Bolt.Sips.Socket.html#recv/3"},{"type":"function","title":"Bolt.Sips.Socket.send/2","doc":"Delegates to :gen_tcp.send/2","ref":"Bolt.Sips.Socket.html#send/2"},{"type":"function","title":"Bolt.Sips.Socket.setopts/2","doc":"Delegates to :inet.setopts/2","ref":"Bolt.Sips.Socket.html#setopts/2"},{"type":"module","title":"Bolt.Sips.Types","doc":"Basic support for representing nodes, relationships and paths belonging to\na Neo4j graph database.\n\nFour supported types of entities:\n\n- Node\n- Relationship\n- UnboundRelationship\n- Path\n\nMore details, about the Bolt protocol, here:\nhttps://github.com/boltprotocol/boltprotocol/blob/master/README.md\n\nAdditionally, since bolt V2, new types appears: spatial and temporal\nThose are not documented in bolt protocol, but neo4j documentation can be found here:\nhttps://neo4j.com/docs/cypher-manual/current/syntax/temporal/\nhttps://neo4j.com/docs/cypher-manual/current/syntax/spatial/\n\nTo work with temporal types, the following Elixir structs are available:\n- Time, DateTime, NaiveDateTime\n- Calendar.DateTime to work with timezone (as string)\n- TimeWithTZOffset, DateTimeWithTZOffset to work with (date)time and timezone offset(as integer)\n- Duration\n\nFor spatial types, you only need Point struct as it covers:\n- 2D point (cartesian or geographic)\n- 3D point (cartesian or geographic)","ref":"Bolt.Sips.Types.html"},{"type":"module","title":"Bolt.Sips.Types.DateTimeWithTZOffset","doc":"Manage a Time and its time zone offset.\n\nThis temporal types hs been added in bolt v2","ref":"Bolt.Sips.Types.DateTimeWithTZOffset.html"},{"type":"function","title":"Bolt.Sips.Types.DateTimeWithTZOffset.create/2","doc":"Create a valid DateTimeWithTZOffset from a NaiveDateTime and offset in seconds","ref":"Bolt.Sips.Types.DateTimeWithTZOffset.html#create/2"},{"type":"function","title":"Bolt.Sips.Types.DateTimeWithTZOffset.format_param/1","doc":"Convert DateTimeWithTZOffset struct in a cypher-compliant  string","ref":"Bolt.Sips.Types.DateTimeWithTZOffset.html#format_param/1"},{"type":"type","title":"Bolt.Sips.Types.DateTimeWithTZOffset.t/0","doc":"","ref":"Bolt.Sips.Types.DateTimeWithTZOffset.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.Duration","doc":"a Duration type, as introduced in bolt V2.\n\nComposed of months, days, seconds and nanoseconds, it can be used in date operations","ref":"Bolt.Sips.Types.Duration.html"},{"type":"function","title":"Bolt.Sips.Types.Duration.create/4","doc":"Create a Duration struct from data returned by Neo4j.\n\nNeo4j returns a list of 4 integers:\n  - months\n  - days\n  - seconds\n  - nanoseconds\n\nStruct elements are computed in a logical way, then for exmple 65 seconds is 1min and 5\nseconds. Beware that you may not retrieve the same data you send!\nNote: days are not touched as they are not a fixed number of days for each month.","ref":"Bolt.Sips.Types.Duration.html#create/4"},{"type":"function","title":"Example: - Bolt.Sips.Types.Duration.create/4","doc":"iex> Duration.create(15, 53, 125, 54)\n    %Bolt.Sips.Types.Duration{\n      days: 53,\n      hours: 0,\n      minutes: 2,\n      months: 3,\n      nanoseconds: 54,\n      seconds: 5,\n      weeks: 0,\n      years: 1\n    }","ref":"Bolt.Sips.Types.Duration.html#create/4-example"},{"type":"function","title":"Bolt.Sips.Types.Duration.format_param/1","doc":"Convert a %Duration in a cypher-compliant string.\nTo know everything about duration format, please see:\nhttps://neo4j.com/docs/cypher-manual/current/syntax/temporal/#cypher-temporal-durations","ref":"Bolt.Sips.Types.Duration.html#format_param/1"},{"type":"type","title":"Bolt.Sips.Types.Duration.t/0","doc":"","ref":"Bolt.Sips.Types.Duration.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.Entity","doc":"base structure for Node and Relationship","ref":"Bolt.Sips.Types.Entity.html"},{"type":"module","title":"Bolt.Sips.Types.Node","doc":"Self-contained graph node.\n\n  A Node represents a node from a Neo4j graph and consists of a\n  unique identifier (within the scope of its origin graph), a list of\n  labels and a map of properties.\n\n  https://github.com/boltprotocol/boltprotocol/blob/master/v1/_serialization.asciidoc#node","ref":"Bolt.Sips.Types.Node.html"},{"type":"type","title":"Bolt.Sips.Types.Node.t/0","doc":"","ref":"Bolt.Sips.Types.Node.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.Path","doc":"Self-contained graph path.\n\n  A Path is a sequence of alternating nodes and relationships corresponding to a\n  walk in the graph. The path always begins and ends with a node.\n  Its representation consists of a list of distinct nodes,\n  a list of distinct relationships and a sequence of integers describing the\n  path traversal\n\n  https://github.com/boltprotocol/boltprotocol/blob/master/v1/_serialization.asciidoc#path","ref":"Bolt.Sips.Types.Path.html"},{"type":"function","title":"Bolt.Sips.Types.Path.graph/1","doc":"represents a traversal or walk through a graph and maintains a direction\nseparate from that of any relationships traversed","ref":"Bolt.Sips.Types.Path.html#graph/1"},{"type":"type","title":"Bolt.Sips.Types.Path.t/0","doc":"","ref":"Bolt.Sips.Types.Path.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.Point","doc":"Manage spatial data introduced in Bolt V2\n\nPoint can be:\n  - Cartesian 2D\n  - Geographic 2D\n  - Cartesian 3D\n  - Geographic 3D","ref":"Bolt.Sips.Types.Point.html"},{"type":"function","title":"Bolt.Sips.Types.Point.create/3","doc":"A 2D point either needs:\n- 2 coordinates and a atom (:cartesian or :wgs_84) to define its type\n- 2 coordinates and a srid (4326 or 7203) to define its type","ref":"Bolt.Sips.Types.Point.html#create/3"},{"type":"function","title":"Examples: - Bolt.Sips.Types.Point.create/3","doc":"iex> Point.create(:cartesian, 10, 20.0)\n    %Bolt.Sips.Types.Point{\n      crs: \"cartesian\",\n      height: nil,\n      latitude: nil,\n      longitude: nil,\n      srid: 7203,\n      x: 10.0,\n      y: 20.0,\n      z: nil\n    }\n    iex> Point.create(4326, 10, 20.0)\n    %Bolt.Sips.Types.Point{\n      crs: \"wgs-84\",\n      height: nil,\n      latitude: 20.0,\n      longitude: 10.0,\n      srid: 4326,\n      x: 10.0,\n      y: 20.0,\n      z: 30.0\n    }","ref":"Bolt.Sips.Types.Point.html#create/3-examples"},{"type":"function","title":"Bolt.Sips.Types.Point.create/4","doc":"Create a 3D point\n\nA 3D point either needs:\n- 3 coordinates and a atom (:cartesian or :wgs_84) to define its type\n- 3 coordinates and a srid (4979 or 9147) to define its type","ref":"Bolt.Sips.Types.Point.html#create/4"},{"type":"function","title":"Examples: - Bolt.Sips.Types.Point.create/4","doc":"iex> Point.create(:cartesian, 10, 20.0, 30)\n    %Bolt.Sips.Types.Point{\n      crs: \"cartesian-3d\",\n      height: nil,\n      latitude: nil,\n      longitude: nil,\n      srid: 9157,\n      x: 10.0,\n      y: 20.0,\n      z: 30.0\n    }\n    iex> Point.create(4979, 10, 20.0, 30)\n    %Bolt.Sips.Types.Point{\n      crs: \"wgs-84-3d\",\n      height: 30.0,\n      latitude: 20.0,\n      longitude: 10.0,\n      srid: 4979,\n      x: 10.0,\n      y: 20.0,\n      z: 30.0\n    }","ref":"Bolt.Sips.Types.Point.html#create/4-examples"},{"type":"function","title":"Bolt.Sips.Types.Point.format_param/1","doc":"Convert a Point struct into a cypher-compliant map","ref":"Bolt.Sips.Types.Point.html#format_param/1"},{"type":"function","title":"Example - Bolt.Sips.Types.Point.format_param/1","doc":"iex(8)> Point.create(4326, 10, 20.0) |> Point.format_to_param\n    %{crs: \"wgs-84\", latitude: 20.0, longitude: 10.0, x: 10.0, y: 20.0}","ref":"Bolt.Sips.Types.Point.html#format_param/1-example"},{"type":"type","title":"Bolt.Sips.Types.Point.t/0","doc":"","ref":"Bolt.Sips.Types.Point.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.Relationship","doc":"Self-contained graph relationship.\n\n  A Relationship represents a relationship from a Neo4j graph and consists of\n  a unique identifier (within the scope of its origin graph), identifiers\n  for the start and end nodes of that relationship, a type and a map of properties.\n\n  https://github.com/boltprotocol/boltprotocol/blob/master/v1/_serialization.asciidoc#relationship","ref":"Bolt.Sips.Types.Relationship.html"},{"type":"type","title":"Bolt.Sips.Types.Relationship.t/0","doc":"","ref":"Bolt.Sips.Types.Relationship.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.TimeWithTZOffset","doc":"Manage a Time and its time zone offset.\n\nThis temporal types hs been added in bolt v2","ref":"Bolt.Sips.Types.TimeWithTZOffset.html"},{"type":"function","title":"Bolt.Sips.Types.TimeWithTZOffset.create/2","doc":"Create a valid TimeWithTZOffset from a Time and offset in seconds","ref":"Bolt.Sips.Types.TimeWithTZOffset.html#create/2"},{"type":"function","title":"Bolt.Sips.Types.TimeWithTZOffset.format_param/1","doc":"Convert TimeWithTZOffset struct in a cypher-compliant  string","ref":"Bolt.Sips.Types.TimeWithTZOffset.html#format_param/1"},{"type":"type","title":"Bolt.Sips.Types.TimeWithTZOffset.t/0","doc":"","ref":"Bolt.Sips.Types.TimeWithTZOffset.html#t:t/0"},{"type":"module","title":"Bolt.Sips.Types.UnboundRelationship","doc":"Self-contained graph relationship without endpoints.\n  An UnboundRelationship represents a relationship relative to a\n  separately known start point and end point.\n\n  https://github.com/boltprotocol/boltprotocol/blob/master/v1/_serialization.asciidoc#unboundrelationship","ref":"Bolt.Sips.Types.UnboundRelationship.html"},{"type":"type","title":"Bolt.Sips.Types.UnboundRelationship.t/0","doc":"","ref":"Bolt.Sips.Types.UnboundRelationship.html#t:t/0"},{"type":"module","title":"Bolt.Sips.TypesHelper","doc":"","ref":"Bolt.Sips.TypesHelper.html"},{"type":"function","title":"Bolt.Sips.TypesHelper.datetime_with_micro/2","doc":"Convert NaiveDateTime and timezone into a Calendar.DateTime\nWithout losing micorsecond data!","ref":"Bolt.Sips.TypesHelper.html#datetime_with_micro/2"},{"type":"function","title":"Bolt.Sips.TypesHelper.decompose_in_hms/1","doc":"Decompose an amount seconds into the tuple {hours, minutes, seconds}","ref":"Bolt.Sips.TypesHelper.html#decompose_in_hms/1"},{"type":"function","title":"Bolt.Sips.TypesHelper.formated_time_offset/1","doc":"Convert an amount of seconds in a +hours:minutes offset","ref":"Bolt.Sips.TypesHelper.html#formated_time_offset/1"},{"type":"module","title":"Bolt.Sips.Utils.Converters","doc":"","ref":"Bolt.Sips.Utils.Converters.html"},{"type":"function","title":"Bolt.Sips.Utils.Converters.to_float/1","doc":"","ref":"Bolt.Sips.Utils.Converters.html#to_float/1"},{"type":"module","title":"Bolt.Sips.Utils.ModuleInspector","doc":"","ref":"Bolt.Sips.Utils.ModuleInspector.html"},{"type":"function","title":"Bolt.Sips.Utils.ModuleInspector.get_modules/0","doc":"","ref":"Bolt.Sips.Utils.ModuleInspector.html#get_modules/0"},{"type":"function","title":"Bolt.Sips.Utils.ModuleInspector.match_modules/1","doc":"","ref":"Bolt.Sips.Utils.ModuleInspector.html#match_modules/1"},{"type":"task","title":"mix bolt.cypher","doc":"Quickly run Cypher commands from a mix task","ref":"Mix.Tasks.Bolt.Cypher.html"},{"type":"task","title":"Command line options - mix bolt.cypher","doc":"- `--url`, `-u` - Neo4j server URL\n  - `--ssl`, `-s` - use ssl\n\nThe command line options have lower precedence than the options\nspecified in your `mix.exs` file, if defined.\n\nExamples:\n\n    MIX_ENV=test mix bolt.cypher \"MATCH (people:Person) RETURN people.name LIMIT 5\"\n\n    Output sample:\n\n    \"MATCH (people:Person) RETURN people.name as name LIMIT 5\"\n    [%{\"name\" => \"Keanu Reeves\"}, %{\"name\" => \"Carrie-Anne Moss\"},\n     %{\"name\" => \"Andy Wachowski\"}, %{\"name\" => \"Lana Wachowski\"},\n     %{\"name\" => \"Joel Silver\"}]","ref":"Mix.Tasks.Bolt.Cypher.html#module-command-line-options"},{"type":"exception","title":"Bolt.Sips.Exception","doc":"This module defines a `Bolt.Sips.Exception` structure containing two fields:\n\n* `code` - the error code\n* `message` - the error details","ref":"Bolt.Sips.Exception.html"},{"type":"type","title":"Bolt.Sips.Exception.t/0","doc":"","ref":"Bolt.Sips.Exception.html#t:t/0"},{"type":"extras","title":"README","doc":"","ref":"readme.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"2.1.0 - Changelog","doc":"Thank you https://github.com/zediogoviana, for the following improvements:\n\n- Add configurable SSL options\n- Fix local connection\n- Keep the :ssl keyword to manage the options also\n\nDependencies updated, paving the road to switching to latest Elixir/Erlang combo.","ref":"changelog.html#2-1-0"},{"type":"extras","title":"2.0.11 - Changelog","doc":"- Issue #100: Timeout set in config in now used by queries\n- DBConnection, bump dependencies","ref":"changelog.html#2-0-11"},{"type":"extras","title":"2.0.10 - Changelog","doc":"- Fix temporal types usage: microseconds are not fully available\n- Review to pass test on Neo4j 4 :\n  - test and doctests to use new parameter syntax (using {} is deprecated in Neo4j 4)\n  - `toUpper` instead of `upper`","ref":"changelog.html#2-0-10"},{"type":"extras","title":"2.0.9 - Changelog","doc":"- fix: (Bolt.Sips.Exception) unable to encode value: -128, see: https://boltprotocol.org/v1/#ints, for details. Closes #93 Thank you, @kalamarski-marcin","ref":"changelog.html#2-0-9"},{"type":"extras","title":"2.0.8 - Changelog","doc":"- Fix Response.profile not being properly filled. Closes #91","ref":"changelog.html#2-0-8"},{"type":"extras","title":"2.0.7 - Changelog","doc":"- sometimes the server version is missing the patch number, and the router couldn't return the proper version. Thank you @barry-w-hill, for finding this bug and reporting it!\n- remove the `basic_auth` when using the `&Bolt.Sips.info/0` function. Thanks @dominique-vassard, for suggestion. Closes: #89","ref":"changelog.html#2-0-7"},{"type":"extras","title":"2.0.6 - Changelog","doc":"- Fix 'unused alias' compilation warnings\n- Fix Bolt.Sips.Response type: `stats` was a `list` instead of `list|map`\n- Add typespec for Bolt.Sips.Types: Node, Relationship and UnboundRelationship","ref":"changelog.html#2-0-6"},{"type":"extras","title":"2.0.5 - Changelog","doc":"- fix #83. More details in commit: https://github.com/florinpatrascu/bolt_sips/commit/ebe17e62ab1d823e301b11d99d532663b0b25135 Thank you @kristofka!","ref":"changelog.html#2-0-5"},{"type":"extras","title":"2.0.4 - Changelog","doc":"- feature: support connection options in queries PR #82. Many thanks @tcrossland, for this contribution!\n  This PR adds support for passing options through to DBConnection.execute/4\n- fix some broken links, in the docs; closes #76\n- update some dependencies, including the DBConnection package.\n- squashing some compile warnings; to be continued /attn: @team ;)\n- please use Elixir 1.9 or 1.10, for test and development - where possible.","ref":"changelog.html#2-0-4"},{"type":"extras","title":"2.0.3 - Changelog","doc":"- refactoring the internals for achieving a better performance, while improving the code readability and extensibility - many thanks to @kristofka and @dominique-vassard. You guys are awesome!\n- fix: Consistent bad connection state after malformed query [...] issue #78\n\n## === 2.0.2 ===\n\n- The 2.0, stable release. Thank you all for your feedback and for contributing to making this driver better. w⦿‿⦿t!\n- fix: Simple Query taking too much time to process #73","ref":"changelog.html#2-0-3"},{"type":"extras","title":"2.0.0-rc.2 - Changelog","doc":"- swapping the assets around, for better organizing the docs","ref":"changelog.html#2-0-0-rc-2"},{"type":"extras","title":"2.0.0-rc.1 - Changelog","doc":"- more documentation\n- fix the TravisCi build\n- min versions\n  erlang 21.2\n  elixir 1.7\n\n## === 2.0.0-rc ===","ref":"changelog.html#2-0-0-rc-1"},{"type":"extras","title":"What's New? - Changelog","doc":"### `bolt+routing://` is now supported\n\nRead more what this schema is, as defined by the [Neo4j team](https://neo4j.com/developer/kb/how-neo4j-browser-bolt-routing/)\n\n#","ref":"changelog.html#what-s-new"},{"type":"extras","title":"Role-based connections - Changelog","doc":"Until this version, Bolt.Sips was used for connecting to a single Neo4j server, aka: the \"direct\" mode. Basically you configure the driver with a url to a Neo4j server and Bolt.Sips will use that to attach itself to it, using a single configuration, remaining attached to that server until it is restarted (or reconfigured). In direct mode, bolt_sips \"knows\" only one server.\n\nStarting with this version you can have as many distinct connection configurations, each of them dedicated to different Neo4j servers, as/if needed. We call these connections: \"role-based connections\". For example, when you'll connect to a Neo4j cluster using the new protocol, i.e. by using a configuration like this:\n\n    config :bolt_sips, Bolt,\n      # default port considered to be: 7687\n      url: \"bolt+routing://localhost\",\n      basic_auth: [username: \"neo4j\", password: \"test\"],\n      pool_size: 10\n\nBolt.Sips will automatically create three pools of size 10, with the following **reserved** names: `:read`, `:write` and `:route`. Now you can specify what type of connection you want to use, by its name (role). For example:\n\n    wconn = Bolt.Sips.conn(:write)\n    ... = Bolt.Sips.query!(wconn, \"CREATE (a:Person {name:'Bob'})\")\n\n    rconn = Bolt.Sips.conn(:read)\n    ... = Bolt.Sips.query!(rconn, \"MATCH (a:Person {name: 'Bob'}) RETURN a.name AS name\")\n\nThe roles above: `:read`, `:write` and `:route`, are reserved. Please do not name custom connections using the same names (atoms). And as you just realized, yes: now you can create as many Bolt.Sips **direct** \"driver instances\" as you want, or as many as your app/hardware supports.\n\nPlease see the documentation for much more details.\n\n#","ref":"changelog.html#role-based-connections"},{"type":"extras","title":"Main breaking changes introduced in version 2.x - Changelog","doc":"- the `hostname` config parameter is a string; used to be a charlist\n- the `url` config parameter must start with a valid schema i.e. `bolt`, `bolt+routing` or `neo4j`.\n  Examples:\n\n      url: \"bolt://localhost\"\n      url: \"bolt+routing://neo4j:password@neo01.graph.example.com:123456?policy=europe\"\n\n- Bolt.Sips.Query, will return a Bolt.Sips.Response now; it used to be a simple data structure.\n\n## === 1.5 ===","ref":"changelog.html#main-breaking-changes-introduced-in-version-2-x"},{"type":"extras","title":"1.5.1 - Changelog","doc":"- add a test alias for running the tests compatible with the most recent Neo4j server while\n  disabling the older/legacy ones\n- cleanup some warning about unused aliases","ref":"changelog.html#1-5-1"},{"type":"extras","title":"1.5.0 - Changelog","doc":"- Bolt V3 support\n- Decompose tests by bolt version\n- Important note about transaction","ref":"changelog.html#1-5-0"},{"type":"extras","title":"1.4.0 - Changelog","doc":"- Encoding / Decoding types is now at the lowest possible level\n- Decompose encoders / decoders by bolt version\n- Expose only public API in docs","ref":"changelog.html#1-4-0"},{"type":"extras","title":"1.3.0 - Changelog","doc":"- 1.3.0 stable release. Many thanks to Dominique VASSARD, for his awesome contributions.","ref":"changelog.html#1-3-0"},{"type":"extras","title":"1.3.0-rc2 - Changelog","doc":"- Fix some typos\n- add json encoding capability","ref":"changelog.html#1-3-0-rc2"},{"type":"extras","title":"1.2.2-rc2 - Changelog","doc":"- Bug fix: Nanoseconds formating was erroneous. Example: 54 nanoseconds was formated to \"PT0.54S\" instead of \"PT0.000000054S\"\n- Bug fix: Large amount of nanoseconds (>= 1_000_000) wasn't treated and lead to Neo4j errors. Now large amount of nanoseconds are converted in seconds, with the remainder in nanoseconds.","ref":"changelog.html#1-2-2-rc2"},{"type":"extras","title":"1.2.1-rc2 - Changelog","doc":"- Bug fix: If a property contains a speciifc types (date, datetime, point, etc.), it wasn't decoded. see: https://github.com/florinpatrascu/bolt_sips/issues/55","ref":"changelog.html#1-2-1-rc2"},{"type":"extras","title":"1.2.0-rc2 - Changelog","doc":"- support for the spatial and temporal types.","ref":"changelog.html#1-2-0-rc2"},{"type":"extras","title":"1.1.0-rc2 - Changelog","doc":"- removed the `boltex` dependency and added all its \"low-level\" code to `internals`.","ref":"changelog.html#1-1-0-rc2"},{"type":"extras","title":"1.0.0-rc2 - Changelog","doc":"#","ref":"changelog.html#1-0-0-rc2"},{"type":"extras","title":"Breaking changes introduced in version 1.x - Changelog","doc":"- non-closure based transactions are not supported anymore. This is a change introduced in DBConnection 2.x. `Bolt.Sips` version tagged `v0.5.10` is the last version supporting open transactions.\n- the support for ETLS was dropped. It was mostly used for development or hand-crafted deployments\n\nThis version is using the official [DBConnection 2.0.0-rc2](https://hex.pm/packages/db_connection/2.0.0-rc.0), from [hex.pm](https://hex.pm)","ref":"changelog.html#breaking-changes-introduced-in-version-1-x"},{"type":"extras","title":"0.5.10 - Changelog","doc":"- update the links referencing the Bolt protocol documentation (types, etc)","ref":"changelog.html#0-5-10"},{"type":"extras","title":"0.5.9 - Changelog","doc":"- upgrade dependencies\n- trading carefully around the new db_connection, as we're chasing the code from `master` currently, and there more changes in the pipe to come for the both projects; db_connection, and this one, respectively.","ref":"changelog.html#0-5-9"},{"type":"extras","title":"0.5.8 - Changelog","doc":"- dealing with negative integers see issue #42, for more details","ref":"changelog.html#0-5-8"},{"type":"extras","title":"0.5.7 - Changelog","doc":"- elixir 1.6 and code formatting, of course :)\n- minor test updates\n- update dependencies\n- pending code for the newest `db_connection` (currently using db_connection from the master branch)","ref":"changelog.html#0-5-7"},{"type":"extras","title":"0.5.5 - Changelog","doc":"- using the [DBConnection](https://hexdocs.pm/db_connection/DBConnection.html), thanks to the work done by Dmitriy Nesteryuk.","ref":"changelog.html#0-5-5"},{"type":"extras","title":"0.4.11 - Changelog","doc":"- using Elixir 1.5\n- not using the ConCache anymore. I initially intended to use its support throughout the driver, but it is not needed.\n- README updated with a short snippet from a Phoenix web app demo, showing how to start Bolt.Sips, as a worker\n- dependencies update\n- minor code cleanup, to prep the code for receiving HA and Bolt routing capabilities","ref":"changelog.html#0-4-11"},{"type":"extras","title":"v0.3.5 - Changelog","doc":"- better error messages; issue #33\n- not retrying a connection when the server is not available/started\n- incorrect number of retries, performed by the driver in case of errors; was one extra","ref":"changelog.html#v0-3-5"},{"type":"extras","title":"v0.3.4 - Changelog","doc":"- dependencies update, minor code cleanup, listening to Credo :) and finally using a Markdown linter","ref":"changelog.html#v0-3-4"},{"type":"extras","title":"v0.3.3 - Changelog","doc":"- Add link to travis build; #31 by vic","ref":"changelog.html#v0-3-3"},{"type":"extras","title":"v0.3.2 - Changelog","doc":"- Use the project's own configuration file when executing the `bolt.cypher` mix task. Fixes issue #20","ref":"changelog.html#v0-3-2"},{"type":"extras","title":"v0.3.1 Breaking changes - Changelog","doc":"- rollback/refactor to optionally allow external configuration options to be defined at runtime. You must start the Bolt.Sips manually, when needed, i.e. `Bolt.Sips.start_link(url: \"localhost\")`, or by changing your app's mix config file, i.e.\n\n```elixir\ndef application do\n  [applications: [:logger, :bolt_sips],\n   mod: {Bolt.Sips.Application, []}]\nend\n```\n\nYou can also specify custom configuration settings in you app's mix config file. These may overwrite your config file:\n\n```elixir\ndef application do\n  [extra_applications: [:logger], mod:\n    {Bolt.Sips.Application, [url: 'localhost', pool_size: 15]}\n  ]\nend\n```\n\n- code cleanup","ref":"changelog.html#v0-3-1-breaking-changes"},{"type":"extras","title":"v0.2.6 (2017-04-21) - Changelog","doc":"- cleanup, and minor dependencies update","ref":"changelog.html#v0-2-6-2017-04-21"},{"type":"extras","title":"v0.2.5 (2017-03-22) - Changelog","doc":"- split multi-line Cypher statements containing semicolons only if the `;` character is at the end of the line, followed by \\r\\n on Windows and \\n on Unix like system, otherwise it may break the Cypher statement when the semicolon appears somewhere else","ref":"changelog.html#v0-2-5-2017-03-22"},{"type":"extras","title":"v0.2.4 (2017-02-26) - Changelog","doc":"- add the fuzzyurl to the list of apps, for project using Elixir < 1.4 (thank you, @dnesteryuk!)","ref":"changelog.html#v0-2-4-2017-02-26"},{"type":"extras","title":"v0.2.3 (2017-02-26) - Changelog","doc":"- improved connection handling","ref":"changelog.html#v0-2-3-2017-02-26"},{"type":"extras","title":"v0.2.2 (2017-02-24) - Changelog","doc":"- PR #18; Bring up `:boltex` and `:retry` in `applications`, for Elixir < 1.4 (from: @wli0503, thank you!)\n- PR #19; test for error message on invalid parameter types (from: @vic, thank you!).","ref":"changelog.html#v0-2-2-2017-02-24"},{"type":"extras","title":"v0.2.1 (2017-02-20) - Changelog","doc":"- stop retrying a request if the failure is an internal one (driver, or driver dependencies related).\n- update the Boltex driver containing two important bug fixes: one where Boltex will fail when receiving too much data (florinpatrascu/bolt_sips/issues/16) and the other one, an improvement, make Boltex.Error.get_id/1 more resilient for new transports (details here: mschae/boltex/issues/14)\n- changed the pool strategy to :fifo, and its timeout to :infinity, and let the (:gen_server) call timeout expire according to the user's :timeout configuration parameter\n- added a test unit provided by @adri (thank you), for executing a Cypher query, with large set of parameters","ref":"changelog.html#v0-2-1-2017-02-20"},{"type":"extras","title":"v0.2.0 Breaking changes - Changelog","doc":"- Elixir 1.4 is now required.\n- Using Boltex 0.2.0\n- bugfix: invalid Cypher statements will now be properly handled when the request is retried automatically","ref":"changelog.html#v0-2-0-breaking-changes"},{"type":"extras","title":"v0.1.11 - Changelog","doc":"- With a larger amount of parameters it seems like generating chunks isn't working correctly. This is a patch imported from Boltex, see: https://github.com/mschae/boltex/issues/13, for more info","ref":"changelog.html#v0-1-11"},{"type":"extras","title":"v0.1.10 (2017-02-11) - Changelog","doc":"- accept Map and Struct for query parameters, transparently. Thank you [@wli0503], for the PR.","ref":"changelog.html#v0-1-10-2017-02-11"},{"type":"extras","title":"v0.1.9 (2017-01-27) - Changelog","doc":"Some of the users are encountering difficulties when trying to compile bolt_sips on Windows. This release is addressing their concern.\n\n`Bolt.Sips` will use the optional System variable: `BOLT_WITH_ETLS`, for depending on the [ETLS](https://hex.pm/packages/etls) package. If that variable is not defined, then `Bolt.Sips` will use the standard Erlang [`:ssl` module](http://erlang.org/doc/man/ssl.html), for the SSL/TLS protocol; the default behavior, starting with this version.\n\nTherefore, if you want the **much** faster ssl/tls support offered by ETLS, then use this: `export BOLT_WITH_ETLS=true` on Linux/OSX, for example. Then:\n\n```elixir\n mix deps.get\n mix test\n```\n\nand so on.\n\n(Don't forget to `mix deps.unlock --all`, if you plan to plan to further debugging/developing w/ or w/o the `BOLT_WITH_ETLS` support)\n\nMany thanks to: [Ben Wilson](https://elixir-lang.slack.com/team/benwilson512), for advices.","ref":"changelog.html#v0-1-9-2017-01-27"},{"type":"extras","title":"v0.1.8 (2017-01-07) - Changelog","doc":"- using Elixir 1.4\n- add more details to the README, about the components required to build ETLS, the TCP/TLS layer\n- added newer Elixirs to the Travis CI configuration file\n- minor code cleanups","ref":"changelog.html#v0-1-8-2017-01-07"},{"type":"extras","title":"v0.1.7 (2017-01-02) - Changelog","doc":"- Connection code refactored for capturing the errors when the remote server is not responding on the first request, or if the driver is misconfigured i.e. wrong port number, bad hostname ...\n- updated the test configuration file with detailed info about the newly introduced option: `:retry_linear_backoff`, mostly as a reminder","ref":"changelog.html#v0-1-7-2017-01-02"},{"type":"extras","title":"v0.1.6 (2017-01-01) - Changelog","doc":"- we're already using configurable timeouts, when executing requests from the connection pool. But with Bolt, the initial handshake sequence (happening before sending any commands to the server) is represented by two important calls, executed in sequence: `handshake` and `init`, and they must both succeed, before sending any (Cypher) requests. You can see the details in the [Bolt protocol](http://boltprotocol.org/v1/#handshake) specs. This sequence is also sensitive to latencies, such as: network latencies, busy servers, etc., and because of that we're introducing a simple support for retrying the handshake (and the subsequent requests) with a linear backoff, and try the handshake sequence (or the request) a couple of times before giving up - all these as part of the exiting pool management, of course. This retry is configurable via a new configuration parameter, the: `:retry_linear_backoff`, respectively. For example:\n\n```elixir\nconfig :bolt_sips, Bolt,\n  url: \"bolt://Bilbo:Baggins@hobby-hobbits.dbs.graphenedb.com:24786\",\n  ssl: true,\n  timeout: 15_000,\n  retry_linear_backoff: [delay: 150, factor: 2, tries: 3]\n```\n\nIn the example above the retry will linearly increase the delay from 150ms following a Fibonacci pattern, cap the delay at 15 seconds (the value defined by the `:timeout` parameter) and giving up after 3 attempts. The same retry mechanism (and configuration parameters) is also honored when we send requests to the neo4j server.","ref":"changelog.html#v0-1-6-2017-01-01"},{"type":"extras","title":"v0.1.5 (2016-12-30) - Changelog","doc":"- as requested by many users, this version is introducing the optional `url` configuration parameter. If present, it will be used for extracting the host name, the port and the authentication details. Please see the README, for a couple of examples. For brevity:\n\n```elixir\nconfig :bolt_sips, Bolt,\n  url: 'bolt://demo:demo@hobby-wowsoeasy.dbs.graphenedb.com:24786',\n  ssl: true\n```","ref":"changelog.html#v0-1-5-2016-12-30"},{"type":"extras","title":"v0.1.4 (Merry Christmas) - Changelog","doc":"- add support for connecting to Neo4j servers on encrypted sockets. Currently only TLSv1.2 is supported, using the default [BoringSSL](https://boringssl.googlesource.com/boringssl/) cipher; via [:etls](https://github.com/kzemek/etls). To connect securely to a remote Neo4j server, such as the ones provided by graphenedb.com, modify your Bolt.Sips config file like this (example):\n\n```elixir\nconfig :bolt_sips, Bolt,\n  hostname: 'bolt://hobby-blah.dbs.graphenedb.com',\n  basic_auth: [username: \"wow\", password: \"of_course_this_is_the_password\"],\n  port: 24786,\n  pool_size: 5,\n  ssl: true,\n  max_overflow: 1\n```\n\nObserve the new flag: `ssl: true`\n\nPlease note this is work in progress","ref":"changelog.html#v0-1-4-merry-christmas"},{"type":"extras","title":"v0.1.2 (2016-11-06) - Changelog","doc":"- integrate the Boltex code from https://github.com/mschae/boltex, and let the Bolt.Sips wrapper to manage the connectivity, using a simple Poolboy implementation for connection pooling","ref":"changelog.html#v0-1-2-2016-11-06"},{"type":"extras","title":"v0.1.1 (2016-09-09) - Changelog","doc":"- a temporary solution for dealing with negative values while extracting a graph walk-through from a Path. Dealing with this in Boltex instead, but this fix should work for now.","ref":"changelog.html#v0-1-1-2016-09-09"},{"type":"extras","title":"v0.1.0 (2016-08-31) - Changelog","doc":"First release!","ref":"changelog.html#v0-1-0-2016-08-31"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nLet's start by creating a simple Elixir project, as a playground for our tests.\n\n```sh\nmix new neo4j_demo --sup --app n4d --module N4D\ncd neo4j_demo\n```\n\nOpen the `mix.exs` and add the bolt_sips dependency.\n\n```elixir\ndefmodule N4D.MixProject do\n  use Mix.Project\n\n  def project do\n    [\n      app: :n4d,\n      version: \"0.1.0\",\n      elixir: \"~> 1.8\",\n      start_permanent: Mix.env() == :prod,\n      deps: deps()\n    ]\n  end\n\n  # Run \"mix help compile.app\" to learn about applications.\n  def application do\n    [\n      extra_applications: [:logger],\n      mod: {N4D.Application, []}\n    ]\n  end\n\n  # Run \"mix help deps\" to learn about dependencies.\n  defp deps do\n    [\n      {:bolt_sips, \"~> 2.0.0-rc\"},\n      {:jason, \"~> 1.1\"}\n    ]\n  end\nend\n```\n\nwe added the [jason](https://hex.pm/packages/jason) library too, for converting the server responses to json. And then run:\n\n```sh\nmix do deps.get, compile\n```","ref":"getting-started.html"},{"type":"extras","title":"Starting the driver - Getting Started","doc":"And our simple project is ready for us to start experimenting with it.\n\nLet's first configure the connection to a running Neo4j server. We presume a standalone community edition server is started and available on the `localhost` interface, and having its Bolt port open at: `7687`. For simplicity edit the `config/config.exs`, and modify it to look like this:\n\n```elixir\nuse Mix.Config\n\nconfig :bolt_sips, Bolt,\n  url: \"bolt://localhost:7687\",\n  basic_auth: [username: \"neo4j\", password: \"test\"],\n  pool_size: 10\n\n```\n\nWith the project configured to connect to a Neo4j server, in direct mode, we can add `Bolt.Sips` to the app's main supervision tree, and let the OTP manage it.\n\n```elixir\n# lib/n4_d/application.ex\n\ndefmodule N4D.Application do\n  @moduledoc false\n\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Bolt.Sips, Application.get_env(:bolt_sips, Bolt)}\n    ]\n\n    opts = [strategy: :one_for_one, name: N4D.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nThere are a couple of different other ways to start the driver but let's keep it simple for now.\n\nThe easiest way to start playing with the driver, in the current configuration, is to drop into the IEx shell and run simple Cypher commands through it.\n\n```sh\ncd neo4j_demo\niex -S mix\n```","ref":"getting-started.html#starting-the-driver"},{"type":"extras","title":"Usage - Getting Started","doc":"A few examples:\n\n```elixir\niex> alias Bolt.Sips, as: Neo\niex> alias Bolt.Sips.Response\n\n# check the driver is up and running:\n\niex> Neo.info()\n%{\n  default: %{\n    connections: %{direct: %{\"localhost:7687\" => 0}, routing_query: nil},\n    user_options: [\n      socket: Bolt.Sips.Socket,\n      port: 7687,\n      url: \"bolt://localhost:7687\",\n      # ...\n      basic_auth: [username: \"neo4j\", password: \"test\"],\n      pool_size: 10\n    ]\n  }\n}\n\n# in direct mode, our current configuration, all the operations such as: read/write or\n# delete, are sent to the Neo4j server using a common connection (pool).\n# Let's obtain a connection:\n\niex> conn = Neo.conn()\n#PID \n\n# a few examples:\n\niex> response = Neo.query!(conn, \"CREATE (p:Person)-[:LIKES]->(t:Technology)\")\n%Response{\n  bookmark: nil,\n  fields: [],\n  notifications: [],\n  plan: nil,\n  profile: nil,\n  records: [],\n  results: [],\n  stats: %{\n    \"labels-added\" => 2,\n    \"nodes-created\" => 2,\n    \"relationships-created\" => 1\n  },\n  type: \"w\"\n}\n\n# query with undirected relationship unless sure of direction\n%Bolt.Sips.Response{results: results} = response =  Neo.query!(conn, \"MATCH (p:Person)-[:LIKES]-(t:Technology) RETURN p\")\n\n# where `results` contain:\n[%{\"p\" => %Bolt.Sips.Types.Node{id: 355, labels: [\"Person\"], properties: %{}}}]\n\n# and we can also encode them to json, as simple as this:\n\niex> Jason.encode!(results)\n\"[{\\\"p\\\":{\\\"id\\\":355,\\\"labels\\\":[\\\"Person\\\"],\\\"properties\\\":{}}}]\"\n\n# of course you can do more:\n\niex> Bolt.Sips.query!(Bolt.Sips.conn(), \"RETURN [10,11,21] AS arr\", %{}, timeout: 19_000) |>\n...> Enum.reduce(0, &(Enum.sum(&1[\"arr\"]) + &2))\n42\n\n# see more examples and the tests, for getting familiar with what is possible.\n\n# Enjoy!\n```\n\nFollow this link: [Cypher Basics](https://neo4j.com/developer/cypher-query-language/), for a gentle introduction to Cypher; Neo4j's query language. Throughout the code snippets we are often using examples copied from the original documentation published by Neo4j, so that you can feel comfortable with them.","ref":"getting-started.html#usage"},{"type":"extras","title":"Configuration","doc":"# Configuration\n\nBolt.Sips can be configured using the well known Mix config files, or by using simple keyword lists.\n\nThis is the most basic configuration:\n\n```elixir\nconfig :bolt_sips, Bolt,\n  url: \"bolt://localhost:7687\"\n```\n\nIt tells Bolt.Sips your Neo4j server is available locally, and it listens on port 7687, expecting bolt commands.\n\nThese are the values you can configure, and their default values:\n\n- `:url`- a full url to pointing to a running Neo4j server. Please remember you must specify the scheme used to connect to the server. Valid schemes:`bolt`,`bolt+routing`and`neo4j` - the last two being used for connecting to a Neo4j causal cluster.\n- `:pool_size` - the size of the connection pool. Default: 15\n- `:timeout` - a connection timeout value defined in milliseconds. Default: 15_000\n- `:ssl`-`true`, if the connection must be encrypted. If the user wants to specify custom ssl options, just pass a keyword list with the options. More info [here](https://erlef.github.io/security-wg/secure_coding_and_deployment_hardening/ssl) Default:`false`\n- `:prefix`- used for differentiating between multiple connections available in the same app. Default:`:default`","ref":"configuration.html"},{"type":"extras","title":"Examples of configurations - Configuration","doc":"Connecting to remote (hosted) Neo4j servers, such as the ones available (also for free) at [Neo4j/Sandbox](https://neo4j.com/sandbox-v2/):\n\n```elixir\nconfig :bolt_sips, Bolt,\n  url: \"bolt:// : \",\n  basic_auth: [username: \"neo4j\", password: \"#######\"]\n  ssl: true # or for example `[verify: :verify_none]` if custom ssl options\n```","ref":"configuration.html#examples-of-configurations"},{"type":"extras","title":"Direct mode - Configuration","doc":"Until this version, `Bolt.Sips` was used for connecting to a single Neo4j server from the moment the hosting app started, until the hosting app was terminated/restarted. This is known as the: `direct` mode. In `direct` mode, the `Bolt.Sips` driver has one configuration describing the connection to a single Neo4j server.\n\nSince this connection mode is well known to our users, we'll not spend time on talking about it. It is sufficient to say that in direct mode, you have one configurable pool of connections, and the settings governing them i.e. timeout,  size, etc., are all about this single connection.\n\nBecause starting with version 2.0 `Bolt.Sips` is supporting a new type of connectivity: `routing`, for connecting to multiple servers or to a Neo4j causal cluster, you must specify the `scheme` in the `url` parameter, of your configuration. Example, for configuring a connection in `direct` mode:\n\n    url: \"bolt://localhost:7687\"\n\nWe'll spend more ink on talking about the `routing` mode, next.","ref":"configuration.html#direct-mode"},{"type":"extras","title":"Routing mode - Configuration","doc":"With the 2.0 version, `Bolt.Sips` is implementing the ability to connect your app to a Neo4j causal cluster. You can read more about this, here: [Neo4j Causal Clustering](https://neo4j.com/docs/operations-manual/current/clustering/introduction/)\n\nThe features of using a causal cluster, in Neo4j's own words:\n\n> Neo4j’s Causal Clustering provides three main features:\n>\n> - Safety: Core Servers provide a fault tolerant platform for transaction processing which will remain available while a simple majority of those Core Servers are functioning.\n> - Scale: Read Replicas provide a massively scalable platform for graph queries that enables very large graph workloads to be executed in a widely distributed topology.\n> - Causal consistency: when invoked, a client application is guaranteed to read at least its own writes.\n\nTo configure `Bolt.Sips` for connecting to a Neo4j Causal Cluster, you only need the specify the appropriate scheme, in the `url` configuration parameter:\n\n    url: \"bolt+routing://localhost:7687\"\n\nor:\n\n    url: \"neo4j://localhost:7687\"\n\nPrefer the latter, since `bolt+routing` appears to be soon deprecated, by Neo4j. We'll use `neo4j://` throughout the docs for referring to the `routing` mode, for brevity. Read more about `routing`, [here](routing.md).","ref":"configuration.html#routing-mode"},{"type":"extras","title":"Role based connections - Configuration","doc":"When we implemented the routing mode, we realized we could extend this ability to letting you define any number of connections, identified by a role name of your choice. For example, say your default configuration for `Bolt.Sips` looks like this:\n\n```elixir\nconfig :bolt_sips, Bolt,\n  url: \"bolt://localhost:7687\",\n  basic_auth: [username: \"neo4j\", password: \"test\"],\n  pool_size: 10,\n  max_overflow: 2,\n```\n\n`Bolt.Sips` will load it by default, when your application starts. And with a configuration like that, the default mode, you will continue to obtain connections using the default `Bolt.Sips.conn()` function.\n\nHowever, if you require to have different connections, say: to a different Neo4j server that has some specific role, you could add a new configuration, for example:\n\n```elixir\nconfig :bolt_sips, :hidden_gems,\n  url: \"bolt://localhost:1234\",\n  pool_size: 50,\n  role: :hidden_gems\n```\n\nYou'd have to load this config separately, after the starting the `Bolt.Sips`driver. Like this:\n\n```elixir\niex> Bolt.Sips.start_link(Application.get_env(:bolt_sips, :hidden_gems))\n{:ok, #PID }\n```\n\nand the you can use connections from this new configuration, as easy as this:\n\n```elixir\niex> conn = Bolt.Sips.conn(:hidden_gems)\n#PID \n```\n\nwhile for obtaing the connections from your default configuration, is business as usual:\n\n```elixir\niex> conn = Bolt.Sips.conn()\n#PID \n```\n\nThe new connection pool is supervised by the main `Bolt.Sips.ConnectionSupervisor`, you don't have to do anythings special for that.\n\n![](assets/role_based_connections.png?raw=true)\n\nIn the final release, we'll add a friendlier api for adding role-based connections. More details about role-based-connections, [here](role-based-connections.md)","ref":"configuration.html#role-based-connections"},{"type":"extras","title":"Multi tenancy - Configuration","doc":"Another important feature of the 2.0 version, is: **multi-tenancy**.\n\nStarting with this version, your app can connect to any number of Neo4j servers, in `direct` mode or `routing`.\n\nWe introduced a new configurable parameter, named: `prefix`.\n\nAt this time, the only way to configure the driver for multi-tenancy, is programmatically, not via the configuration file. Example:\n\n```elixir\nmy_secret_cluster_config [\n    url: \"neo4j://localhost:9001\",\n    basic_auth: [username: \"neo4j\", password: \"test\"],\n    pool_size: 10,\n    max_overflow: 2,\n    queue_interval: 500,\n    queue_target: 1500,\n    prefix: :secret_cluster\n  ]\n\n{:ok, _pid} = Bolt.Sips.start_link(@routing_connection_config)\nconn = Bolt.Sips.conn(:write, prefix: :secret_cluster)\n```\n\nAnd you can start as many connections as needed, for as long as the `:prefix` has different names. These connections can be used for connecting to the same or different Neo4j servers.\n\nMore details about multi-tenancy, [here](multi-tenancy.md)","ref":"configuration.html#multi-tenancy"},{"type":"extras","title":"Using Bolt.Sips to query the Neo4j server","doc":"# Using Bolt.Sips to query the Neo4j server\n\nLet's talk about the basics of querying a Neo4j server, using `Bolt.Sips`, and a few methods you could use for using the data returned by the server, using the `Bolt.Sips.Response`. You can learn so much more from the official docs, available at [Neo4j](https://neo4j.com/developer/graph-database/), you should start from there, if you want to get a deep understanding about the Neo4j graph database and its query language: Cypher.","ref":"using-cypher.html"},{"type":"extras","title":"What you need? - Using Bolt.Sips to query the Neo4j server","doc":"- a [Neo4j](https://neo4j.com/download/) server running locally and available at this `url`: `bolt://neo4j:test@localhost`\n- a mix project with `:bolt_sips` available","ref":"using-cypher.html#what-you-need"},{"type":"extras","title":"Simple queries, using Cypher - Using Bolt.Sips to query the Neo4j server","doc":"With the above prerequisites, let's drop into the IEx shell and start experimenting.\n\n```sh\ncd my_neo4j\niex -S mix\nErlang/OTP 21 [erts-10.2.3] [source] [64-bit] [smp:8:8] [ds:8:8:10] [async-threads:1] [hipe]\n\nInteractive Elixir (1.8.1) - press Ctrl+C to exit (type h() ENTER for help)\n\niex>\n```\n\nFirst we need to start the driver with a minimalist configuration (unless it is already started by your project?):\n\n```elixir\niex> {:ok, _neo} = Bolt.Sips.start_link(url: \"bolt://neo4j:test@localhost\")\n{:ok, #PID }\niex>\n\n```\n\nPresuming your database is empty, you can still test your setup by running a simple Cypher query:\n\n```elixir\niex> conn = Bolt.Sips.conn()\n#PID \niex> Bolt.Sips.query!(conn, \"RETURN 1 as n\") |>\n...> Bolt.Sips.Response.first()\n%{\"n\" => 1}\n```\n\nand we obtained our first response from the server: `%{\"n\" => 1}`, w⦿‿⦿t!! Now let's try some more complicated Cypher queries. We'll use examples that you may want to paste them in your `.exs/.ex` files rather than into the IEx shell, for readability.\n\nWhile most of the Cypher querier fit on a simple row, and they look compact, you might encounter situations where you need to send multiple queries in a single trip, to the server. `Bolt.Sips` allows you do that.\n\nLet's initialize our **test** database with some data.\n\n```elixir\ncypher = \"\"\"\n  CREATE (BoltSips:BoltSips {title:'Elixir sipping from Neo4j, using Bolt', released:2016, license:'MIT', bolt_sips: true})\n  CREATE (TNOTW:Book {title:'The Name of the Wind', released:2007, genre:'fantasy', bolt_sips: true})\n  CREATE (Patrick:Person {name:'Patrick Rothfuss', bolt_sips: true})\n  CREATE (Kvothe:Person {name:'Kote', bolt_sips: true})\n  CREATE (Denna:Person {name:'Denna', bolt_sips: true})\n  CREATE (Chandrian:Deamon {name:'Chandrian', bolt_sips: true})\n\n  CREATE\n    (Kvothe)-[:ACTED_IN {roles:['sword fighter', 'magician', 'musician']}]->(TNOTW),\n    (Denna)-[:ACTED_IN {roles:['many talents']}]->(TNOTW),\n    (Chandrian)-[:ACTED_IN {roles:['killer']}]->(TNOTW),\n    (Patrick)-[:WROTE]->(TNOTW)\n\"\"\"\n\n{:ok, response} =\n  Bolt.Sips.conn()\n  |> Bolt.Sips.query(cypher)\n```\n\nAccording to the response from the server, this is what we did:\n\n```elixir\niex> response\n%Bolt.Sips.Response{\n  results: [],\n  stats: %{\n    \"labels-added\" => 6,\n    \"nodes-created\" => 6,\n    \"properties-set\" => 19,\n    \"relationships-created\" => 4\n  },\n  type: \"w\"\n}\n```\n\nwe have 6 new Nodes, 6 new labels and 4 new relationships.\n\nAt any time, if you want to clean up the data we're creating, you can use this query:\n\n`MATCH (n {bolt_sips: true}) OPTIONAL MATCH (n)-[r]-() DELETE n,r`\n\nObserve we're adding a `bolt_sips` property to the Nodes we're adding, so that it's easier to refer them in our tests.\n\nLet's see how many nodes of \"type\" (`label`, according to Cypher's official terminology) `Person` having the property `bolt_sips` true, we have in our database:\n\n```elixir\niex> query = \"\"\"\n...>   MATCH (n:Person {bolt_sips: true})\n...>   RETURN n.name AS Name\n...>   ORDER BY Name DESC\n...>   LIMIT 5\n...>  \"\"\"\n\niex> %Bolt.Sips.Response{} = response = Bolt.Sips.query!(conn, query)\n%Bolt.Sips.Response{\n  bookmark: \"neo4j:bookmark:v1:tx21613\",\n  fields: [\"Name\"],\n  notifications: [],\n  plan: nil,\n  profile: nil,\n  records: [[\"Patrick Rothfuss\"], [\"Kote\"], [\"Denna\"]],\n  results: [\n    %{\"Name\" => \"Patrick Rothfuss\"},\n    %{\"Name\" => \"Kote\"},\n    %{\"Name\" => \"Denna\"}\n  ],\n  stats: [],\n  type: \"r\"\n}\n```\n\nWe have 3 of them, and we're only showing the `name` property! Above you see the full `Bolt.Sips.Response` returned by our driver based on the raw data returned by the Neo4j server. The `:results` key, contains the aggregated response you will use most of the time, and for that the `Bolt.Sips.Response` module has some useful helpers, for example:\n\n```elixir\niex> response |>\n...> Bolt.Sips.Response.first()\n%{\"Name\" => \"Patrick Rothfuss\"}\n```\n\nand much more. Check the `Bolt.Sips.Response`'s own docs, for more.","ref":"using-cypher.html#simple-queries-using-cypher"},{"type":"extras","title":"Using temporal and spatial types","doc":"# Using temporal and spatial types\n\nTemporal and spatial types are supported since Neo4J 3.4.\nYou can used the elixir structs: Time, NaiveDateTime, DateTime,\nas well as the Bolt Sips structs: DateTimeWithTZOffset, TimeWithTZOffset, Duration, Point.\n\n```elixir\n$ MIX_ENV=test iex -S mix\nErlang/OTP 21 [erts-10.0.5] [source] [64-bit] [smp:8:8] [ds:8:8:10] [async-threads:1] [hipe]\n\nInteractive Elixir (1.7.3) - press Ctrl+C to exit (type h() ENTER for help)\niex> alias Bolt.Sips.Types.{Duration, DateTimeWithTZOffset, Point, TimeWithTZOffset}\n[Bolt.Sips.Types.Duration, Bolt.Sips.Types.DateTimeWithTZOffset,\n Bolt.Sips.Types.Point, Bolt.Sips.Types.TimeWithTZOffset]\n\niex> alias Bolt.Sips.TypesHelper\nBolt.Sips.TypesHelper\n\niex> {:ok, pid} = Bolt.Sips.start_link(url: \"localhost\", basic_auth: [username: \"neo4j\", password: \"test\"])\n{:ok, #PID }\n\niex> conn = Bolt.Sips.conn\n:bolt_sips_pool\n\n# Date without timezone with Date\niex(8)> Bolt.Sips.query!(conn, \"RETURN date($d) AS d\", %{d: ~D[2019-02-04]})\n[%{\"d\" => ~D[2019-02-04]}]\n\n# Time without timezone with Time\niex> Bolt.Sips.query!(conn, \"RETURN localtime($t) AS t\", %{t: ~T[13:26:08.543440]})\n[%{\"t\" => ~T[13:26:08.543440]}]\n\n# Datetime without timezone with Naive DateTime\niex> Bolt.Sips.query!(conn, \"RETURN localdatetime($ldt) AS ldt\", %{ldt: ~N[2016-05-24 13:26:08.543]})\n[%{\"ldt\" => ~N[2016-05-24 13:26:08.543]}]\n\n# Datetime with timezone ID with DateTime (through Calendar)\niex> date_time_with_tz_id = TypesHelper.datetime_with_micro(~N[2016-05-24 13:26:08.543], \"Europe/Paris\")\n#DateTime \niex> Bolt.Sips.query!(conn, \"RETURN datetime($dt) AS dt\", %{dt: date_time_with_tz_id})\n[%{\"dt\" => #DateTime }]\n\n# Datetime with timezone offset (seconds) with DateTimeWithTZOffset\niex(17)> date_time_with_tz = DateTimeWithTZOffset.create(~N[2016-05-24 13:26:08.543], 7200)\n%Bolt.Sips.Types.DateTimeWithTZOffset{\n  naive_datetime: ~N[2016-05-24 13:26:08.543],\n  timezone_offset: 7200\n}\niex(18)> Bolt.Sips.query!(conn, \"RETURN datetime($dt) AS dt\", %{dt: date_time_with_tz})\n[\n  %{\n    \"dt\" => %Bolt.Sips.Types.DateTimeWithTZOffset{\n      naive_datetime: ~N[2016-05-24 13:26:08.543],\n      timezone_offset: 7200\n    }\n  }\n]\n\n\n# Datetime with timezone offset (seconds) with TimeWithTZOffset\niex> time_with_tz = TimeWithTZOffset.create(~T[12:45:30.250000], 3600)\n%Bolt.Sips.Types.TimeWithTZOffset{\n  time: ~T[12:45:30.250000],\n  timezone_offset: 3600\n}\niex> Bolt.Sips.query!(conn, \"RETURN time($t) AS t\", %{t: time_with_tz})\n[\n  %{\n    \"t\" => %Bolt.Sips.Types.TimeWithTZOffset{\n      time: ~T[12:45:30.250000],\n      timezone_offset: 3600\n    }\n  }\n]\n\n# Cartesian 2D point with Point\niex> point_cartesian_2D = Point.create(:cartesian, 50, 60.5)\n%Bolt.Sips.Types.Point{\n  crs: \"cartesian\",\n  height: nil,\n  latitude: nil,\n  longitude: nil,\n  srid: 7203,\n  x: 50.0,\n  y: 60.5,\n  z: nil\n}\niex> Bolt.Sips.query!(conn, \"RETURN point($pt) AS pt\", %{pt: point_cartesian_2D})\n[\n  %{\n    \"pt\" => %Bolt.Sips.Types.Point{\n      crs: \"cartesian\",\n      height: nil,\n      latitude: nil,\n      longitude: nil,\n      srid: 7203,\n      x: 50.0,\n      y: 60.5,\n      z: nil\n    }\n  }\n]\n\n# Geographic 2D point with Point\niex> point_geo_2D = Point.create(:wgs_84, 50, 60.5)\n%Bolt.Sips.Types.Point{\n  crs: \"wgs-84\",\n  height: nil,\n  latitude: 60.5,\n  longitude: 50.0,\n  srid: 4326,\n  x: 50.0,\n  y: 60.5,\n  z: nil\n}\niex> Bolt.Sips.query!(conn, \"RETURN point($pt) AS pt\", %{pt: point_geo_2D})\n[\n  %{\n    \"pt\" => %Bolt.Sips.Types.Point{\n      crs: \"wgs-84\",\n      height: nil,\n      latitude: 60.5,\n      longitude: 50.0,\n      srid: 4326,\n      x: 50.0,\n      y: 60.5,\n      z: nil\n    }\n  }\n]\n\n# Cartesian 3D point with Point\niex> point_cartesian_3D = Point.create(:cartesian, 50, 60.5, 12.34)\n%Bolt.Sips.Types.Point{\n  crs: \"cartesian-3d\",\n  height: nil,\n  latitude: nil,\n  longitude: nil,\n  srid: 9157,\n  x: 50.0,\n  y: 60.5,\n  z: 12.34\n}\niex> Bolt.Sips.query!(conn, \"RETURN point($pt) AS pt\", %{pt: point_cartesian_3D})\n[\n  %{\n    \"pt\" => %Bolt.Sips.Types.Point{\n      crs: \"cartesian-3d\",\n      height: nil,\n      latitude: nil,\n      longitude: nil,\n      srid: 9157,\n      x: 50.0,\n      y: 60.5,\n      z: 12.34\n    }\n  }\n]\n\n# Geographic 2D point with Point\niex> point_geo_3D = Point.create(:wgs_84, 50, 60.5, 12.34)\n%Bolt.Sips.Types.Point{\n  crs: \"wgs-84-3d\",\n  height: 12.34,\n  latitude: 60.5,\n  longitude: 50.0,\n  srid: 4979,\n  x: 50.0,\n  y: 60.5,\n  z: 12.34\n}\niex> Bolt.Sips.query!(conn, \"RETURN point($pt) AS pt\", %{pt: point_geo_2D})\n[\n  %{\n    \"pt\" => %Bolt.Sips.Types.Point{\n      crs: \"wgs-84\",\n      height: nil,\n      latitude: 60.5,\n      longitude: 50.0,\n      srid: 4326,\n      x: 50.0,\n      y: 60.5,\n      z: nil\n    }\n  }\n]\n```","ref":"using-temporal-and-spatial-types.html"},{"type":"extras","title":"About transactions","doc":"# About transactions\n\nTransaction management in Neo4j 3.5+ differs from what it was in prior versions.\nThe cypher keyword `BEGIN`, `COMMIT` and `ROLLBACK` are no longer available.\n\nIn order to have a query that runs fine in all versions, you should use the following pattern:\n\n```elixir\n# Commit is performed automatically if everythings went fine\nconn = Bolt.Sips.conn()\nBolt.Sips.transaction(conn, fn conn ->\n  result = Bolt.Sips.query!(conn, \"CREATE (m:Movie {title: \"Matrix\"}) RETURN m\")\nend)\n\n# Rollback is performed automatically in case of error\nBolt.Sips.transaction(conn, fn conn ->\n  result =Bolt.Sips.query!(conn, \"Invalid query\")\nend)\n\n# Rollback can stil be forced\nBolt.Sips.transaction(conn, fn conn ->\n  result = Bolt.Sips.query!(conn, \"CREATE (m:Movie {title: \"Matrix\"}) RETURN m\")\n  Bolt.Sips.rollback(conn, :dont_save)\nend)\n```","ref":"about-transactions.html"},{"type":"extras","title":"About encoding","doc":"# About encoding\n\nBolt.Sips provides support for encoding your query result in different formats.\nFor now, only JSON is supported.\n\nThere is two way of encoding data to json:\n\n- By using the helpers provided by the module `Bolt.Sips.ResponseEncoder`\n- Using your usual JSON encoding library. `Bolt.Sips` have implementation for: Jason and Poison. With this the query results can be automatically encoded by one of the libraries available: Jason or Poison. No further work is required when using a framework like: Phoenix, for example.\n\nA few examples around the encoding suport:\n\n```elixir\niex> query_result = [\n   %{\n     \"t\" => %Bolt.Sips.Types.Node{\n       id: 26,\n       labels: [\"Test\"],\n       properties: %{\n         \"created_at\" => \"2019-08-03T12:34:56+01:00\",\n         \"name\" => \"A test node\",\n         \"uid\" => 12345\n       }\n     }\n   }\n ]\n\n# Using Bolt.Sips.ResponseEncoder\n iex> Bolt.Sips.ResponseEncoder.encode(query_result, :json)\n{:ok,\n \"[{\\\"t\\\":{\\\"id\\\":26,\\\"labels\\\":[\\\"Test\\\"],\\\"properties\\\":{\\\"created_at\\\":\\\"2019-08-03T12:34:56+01:00\\\",\\\"name\\\":\\\"A test node\\\",\\\"uid\\\":12345}}}]\"}\niex(11)> Bolt.Sips.ResponseEncoder.encode!(query_result, :json)\n\"[{\\\"t\\\":{\\\"id\\\":26,\\\"labels\\\":[\\\"Test\\\"],\\\"properties\\\":{\\\"created_at\\\":\\\"2019-08-03T12:34:56+01:00\\\",\\\"name\\\":\\\"A test node\\\",\\\"uid\\\":12345}}}]\"\n\n# Using Jason\niex(14)> Jason.encode!(query_result)\n\"[{\\\"t\\\":{\\\"id\\\":26,\\\"labels\\\":[\\\"Test\\\"],\\\"properties\\\":{\\\"created_at\\\":\\\"2019-08-03T12:34:56+01:00\\\",\\\"name\\\":\\\"A test node\\\",\\\"uid\\\":12345}}}]\"\n\n# Using Poison\niex(13)> Poison.encode!(query_result)\n\"[{\\\"t\\\":{\\\"properties\\\":{\\\"uid\\\":12345,\\\"name\\\":\\\"A test node\\\",\\\"created_at\\\":\\\"2019-08-03T12:34:56+01:00\\\"},\\\"labels\\\":[\\\"Test\\\"],\\\"id\\\":26}}]\"\n```\n\nBoth solutions rely on protocols, then they can be easily overridden if needed.\nMore info in the modules `Bolt.Sips.ResponseEncoder.Json`, `Bolt.Sips.ResponseEncoder.Json.Jason`, `Bolt.Sips.ResponseEncoder.Json.Poison`","ref":"about-encoding.html"},{"type":"extras","title":"Routing","doc":"# Routing\n\nWhen connecting to a Neo4j cluster, `Bolt.Sips` will create 3 distinct connection pools, each of them dedicated to one of the following connection types (**connection roles**):\n\n- `:route` - used for getting information from the Neo4j router, such as: routing details about which server is handling what type of role: read/write, and more.\n- `:read` - used for read-only connections\n- `:write` - used for write-only connections.\n\nHaving the `Bolt.Sips` configured in `routing` mode, will enforce your code to clarify what type of connections you want, type you **must** specify when requesting a `Bolt.Sips` connection. Example:\n\n```elixir\nrconn = Bolt.Sips.conn(:read)\nwconn = Bolt.Sips.conn(:write)\nrouter_conn = Bolt.Sips.conn(:route)\n```\n\nWithout being explicit about the connection type, you will receive errors, in case you'll attempt to execute a query that will say: create new nodes, on a server having the role: `read` or `route`. This is the only rule you must observe, when using the `Bolt.Sips` driver with a causal cluster.","ref":"routing.html"},{"type":"extras","title":"Routing walk-through - Routing","doc":"Let's walk-through a simple experiment with using `Bolt.Sips` in routing mode and a Neo4j cluster.\n\nIf you don't have a local server, or a remote Neo4j cluster available for your tests, you can easily setup your own local playground. All you need is Docker.\n\nWe'll use a [docker-compose.yml](../../docker-compose.yml) file that you can find in the `Bolt.Sips` main source repo.\n\nIf you have:\n\n- [Docker]( ) installed, and running. You can get Docker from here: https://docs.docker.com/installation/\n- and a simple Elixir project having the `:bolt_sips` driver installer, as a dependency\n\n#","ref":"routing.html#routing-walk-through"},{"type":"extras","title":"Start the Neo4j cluster - Routing","doc":"In a folder where you have the `docker-compose.yml` file, start a new shell session and run the following command:\n\n    docker-compose up\n\nIf this is the first time you run this command, or use Neo4j as a Docker image, then based on the quality of your Internet connection, you'll wait a few seconds while Docker downloads a Neo4j Enterprise image. You'll see something like this:\n\n```sh\nCreating network \"neo4j_lan\" with the default driver\nPulling core1 (neo4j:3.5.3-enterprise)...\n3.5.3-enterprise: Pulling from library/neo4j\ne7c96db7181b: Pull complete\nf910a506b6cb: Pull complete\nb6abafe80f63: Pull complete\nb95a7fd32595: Pull complete\n6c09128ad074: Pull complete\n648805e5f471: Pull complete\ne2790f69a70d: Pull complete\nCreating core2 ... done\nCreating core3 ... done\nCreating core1 ... done\nAttaching to core3, core1, core2\ncore3    | Changed password for user 'neo4j'.\ncore1    | Changed password for user 'neo4j'.\ncore2    | Changed password for user 'neo4j'.\ncore3    | Active database: graph.db\ncore3    | Directories in use:\ncore3    |   home:         /var/lib/neo4j\ncore3    |   config:       /var/lib/neo4j/conf\n...\n```\n\nand towards the end of the starting sequence, this:\n\n```sh\ncore2    | 2019-06-17 12:37:59.078+0000 INFO  Remote interface available at http://localhost:7475/\ncore3    | 2019-06-17 12:37:59.165+0000 INFO  Remote interface available at http://localhost:7476/\n```\n\nCheck to see if you can connect to your local Neo4j cluster, as simple as pointing your Internet browser to this url: `http://localhost:7474`, and if everything was executed successfully, you'll be seeing the familiar Neo4j web interface.\n\nNow let's play with the `Bolt.Sips`driver and our local Neo4j cluster.\n\nChange your elixir test project configuration and modify the `config/config.exs` file like this (excerpt):\n\n```elixir\nuse Mix.Config\n\nconfig :bolt_sips, Bolt,\n  # bolt+routing will be deprecated?!\n  # url: \"bolt+routing://localhost:7687\",\n  url: \"neo4j://localhost:7687\",\n  basic_auth: [username: \"neo4j\", password: \"test\"],\n  pool_size: 10\n```\n\nthen start a IEx shell session, from the projects'r main folder: `iex -S mix`. While inside the IEx session, let's see if our configuration is sound?\n\n```elixir\niex> Bolt.Sips.info()\n%{\n  default: %{\n    connections: %{\n      read: %{\"localhost:7688\" => 0, \"localhost:7689\" => 0},\n      route: %{\n        \"localhost:7687\" => 0,\n        \"localhost:7688\" => 0,\n        \"localhost:7689\" => 0\n      },\n      write: %{\"localhost:7687\" => 0},\n      routing_query: %{...},\n      ttl: 300,\n      updated_at: 1560775628\n    },\n    user_options: [\n      url: \"neo4j://localhost:7687\",\n      pool_size: 10,\n      ....\n    ]\n  }\n}\n```\n\nif you see the response above, it means your settings are ready. Without going into much details about the data structure above, the routing details are these:\n\n```elixir\n  read: %{\"localhost:7688\" => 0, \"localhost:7689\" => 0},\n  write: %{\"localhost:7687\" => 0},\n  route: %{\n    \"localhost:7687\" => 0,\n    \"localhost:7688\" => 0,\n    \"localhost:7689\" => 0\n  ttl: 300,\n  updated_at: ...\n```\n\nAccording to the routing information returned by our cluster, we have:\n\n- two nodes accepting `:read` commands: `localhost:7688` and`localhost:7689`\n- three nodes capabale of responding with routing specific details: `localhost:7687` `localhost:7688` and `localhost:7689`\n- one node accepting `:write` commands; the `localhost:7687`, respectively.\n\nBut don't worry about the gory details, we got you covered :)\n\nLet's run some Cypher queries.\n\n```elixir\niex> alias Bolt.Sips.Response\niex> alias Bolt.Sips, as: Neo\n\n# obtaining a read(only) connection:\niex> rconn = Neo.conn(:read)\n#PID \n\n# checking if there are any Person nodes \"named\": Bob?\niex> %Response{results: r} = Neo.query!(rconn, \"MATCH (p:Person{name: 'Bob'}) RETURN p\")\n%Bolt.Sips.Response{\n  bookmark: \"neo4j:bookmark:v1:tx2\",\n  fields: [\"p\"],\n  notifications: [],\n  plan: nil,\n  profile: nil,\n  records: [],\n  results: [],\n  stats: [],\n  type: \"r\"\n}\n\n# r is [], meaning: our query found none. So let's create one.\n# First we obtain a connection suitable for `write` operations:\niex> wconn = Neo.conn(:write)\n#PID \n\n# and now we can use it for creating a new node:\n\niex> %Response{results: r} = Neo.query!(wconn, \"CREATE (p:Person{name:'Bob'})\")\n%Bolt.Sips.Response{\n  ...\n  stats: %{\"labels-added\" => 1, \"nodes-created\" => 1, \"properties-set\" => 1},\n  type: \"w\"\n}\n\n# our node was created and has one property set,  w⦿‿⦿t!\n# but can we find it? Rerun the previous query using the `read` connection:\n\niex> Neo.query!(rconn, \"MATCH (p:Person{name: 'Bob'}) RETURN p\") |> Response.first()\n%{\n  \"p\" => %Bolt.Sips.Types.Node{\n    id: 20,\n    labels: [\"Person\"],\n    properties: %{\"name\" => \"Bob\"}\n  }\n}\n\n# and yessss, our new Person node is in the cluster!\n# Do you need its json form, instead? Easy:\niex> Neo.query!(rconn, \"MATCH (p:Person{name: 'Bob'}) RETURN p\") |>\n...> Response.first() |>\n...> Bolt.Sips.ResponseEncoder.encode!(:json)\n\"{\\\"p\\\":{\\\"id\\\":20,\\\"labels\\\":[\\\"Person\\\"],\\\"properties\\\":{\\\"name\\\":\\\"Bob\\\"}}}\"\n\n```\n\nBut what happens if we try to create a new Person, using our `read` connection?\n\n```elixir\niex> Neo.query!(rconn, \"CREATE (p:Person{name:'Alice'})\")\n** (Bolt.Sips.Exception) ... No write operations are allowed directly on this database. Writes must pass through the leader. The role of this server is: FOLLOWER\n```\n\nNeo4j will promptly let us know we can't use that connection for write operations. This is the main difference that you must consider when coding.\n\nSame command executed on the proper (write) connection, will be successful:\n\n```elixir\niex> Neo.query!(wconn, \"CREATE (p:Person{name:'Alice'})\")\n%Bolt.Sips.Response{\n  ...\n  stats: %{\"labels-added\" => 1, \"nodes-created\" => 1, \"properties-set\" => 1},\n  type: \"w\"\n}\n```","ref":"routing.html#start-the-neo4j-cluster"},{"type":"extras","title":"Multi tenancy","doc":"# Multi tenancy\n\nVery similar to the role-based connections, with multi-tenancy you will be able to connect to servers where the type of the server (role) is defined by the server itself, such as the Neo4j causal cluster. This setting is sill in its infancy, it works, but you'll have to be careful when using it.\n\nFor differentiating about Neo4j tenants, we introduced a new configurations parameter, named: `:prefix`. Example:\n\n```elixir\nmonster_cluster_conf = [\n  url: \"neo4j://localhost\",\n  basic_auth: [username: \"neo4j\", password: \"password\"],\n  pool_size: 50,\n  prefix: :monster_cluster\n\nbaby_monster_cluster_conf = [\n  url: \"neo4j://raspberry_π\",\n  basic_auth: [username: \"πs\", password: \"4VR\"],\n  pool_size: 50,\n  prefix: :baby_monster_cluster\n```\n\nIn the example above we defined two different connections, each of them pointing to different Neo4j clusters. As you know now, every cluster will have role-specific connections as defined by the routers, in those clusters. The connection roles will be: `:write`, `:read` and `:route`. To specify what connection you want and on what server, you will use the `:prefix` optional parameter of the new `Bolt.Sips.conn/2` method. Example:\n\n```elixir\nBolt.Sips.conn(:read, prefix: :monster_cluster)\n  |> Bolt.Sips.query!(\"MATCH (n) RETURN n.name AS name\")\n```\n\nor:\n\n```elixir\nBolt.Sips.conn(:read, prefix: :baby_monster_cluster)\n  |> Bolt.Sips.query!(\"MATCH (n) RETURN n.name AS name\")\n```\n\n(wip)","ref":"multi-tenancy.html"},{"type":"extras","title":"Using Bolt.Sips with Phoenix, or similar","doc":"# Using Bolt.Sips with Phoenix, or similar\n\nDon't forget to start the `Bolt.Sips` driver in your supervision tree. Example:\n\n```elixir\ndefmodule MoviesElixirPhoenix do\n  use Application\n\n  # See https://hexdocs.pm/elixir/Application.html\n  # for more information on OTP Applications\n  def start(_type, _args) do\n    # Define workers and child supervisors to be supervised\n    children = [\n      # Start the endpoint when the application starts\n      {Bolt.Sips, Application.get_env(:bolt_sips, Bolt)},\n      %{\n        id: MoviesElixirPhoenix.Endpoint,\n        start: {MoviesElixirPhoenix.Endpoint, :start_link, []}\n      }\n    ]\n\n    # See https://hexdocs.pm/elixir/Supervisor.html\n    # for other strategies and supported options\n    opts = [strategy: :one_for_one, name: MoviesElixirPhoenix.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  # Tell Phoenix to update the endpoint configuration\n  # whenever the application is updated.\n  def config_change(changed, _new, removed) do\n    MoviesElixirPhoenix.Endpoint.config_change(changed, removed)\n    :ok\n  end\nend\n```\n\nThe code above was extracted from [the Neo4j Movies Demo](https://github.com/florinpatrascu/bolt_movies_elixir_phoenix), a Phoenix web application using this driver and the well known [Dataset - Movie Database](https://neo4j.com/developer/movie-database/).\n\nNote: as explained below, you don't need to convert your query result before having it  encoded in JSON. BoltSips provides Jason and Poison implementation to tackle this problem automatically.","ref":"using-with-phoenix.html"}],"content_type":"text/markdown"}